import 'package:dartxx/dartxx.dart';
import 'package:flutter/material.dart';
import 'package:sunny_dart/typedefs.dart';
import 'package:sunny_forms/notifier.dart';
import 'package:sunny_forms/sunny_form_controller.dart';
import 'package:sunny_forms/sunny_form_field.dart';

/// An extension that allows for simple binding to [MModel] instances
class AttributeFormField<T extends Object> extends SunnyFormField<T> {
  AttributeFormField({
    Key? key,
    dynamic label,
    dynamic placeholder,
    dynamic attribute,
    InputDecoration? decoration,
    FormController? form,
    T? initialValue,
    FormValueChanged<T?>? onChange,
    required bool isRequired,

    /// This builder should only build the raw input, without any extra stuff, like errors, help text,
    /// labels, or otherwise.  By default, a basic text field will be created
    SunnyFormFieldBuilder<T?>? builder,
    bool? isReadOnly,
    bool captureNestedPaths = true,
    bool? enabled,
    AttributeAccepter<T?>? onReadOnlyTapped,
    FieldConverter<T?>? converter,
    FocusNode? focusNode,
  }) : super(
          key: key,
          label: label,
          placeholder: placeholder,
          attribute: attribute,
          builder: builder,
          isReadOnly: isReadOnly,
          form: form,
          isRequired: isRequired,
          onChange: onChange,
          enabled: enabled,
          decoration: decoration,
          captureNestedPaths: captureNestedPaths,
          onReadOnlyTapped: onReadOnlyTapped,
          initialValue: initialValue,
//          initialValue: initialValue ??
//              converter?.call(controller.get(JsonPath.of(attribute))) ??
//              controller.get(JsonPath.of(attribute)),
          converter: converter,
          focusNode: focusNode,
        );

  /// Creates an attribute form field, pulling initial value from the form
  AttributeFormField.ofForm({
    Key? key,
    required FormController form,
    dynamic label,
    dynamic placeholder,
    dynamic attribute,
    InputDecoration? decoration,
    FormValueChanged<T?>? onChange,
    required bool isRequired,

    /// This builder should only build the raw input, without any extra stuff, like errors, help text,
    /// labels, or otherwise.  By default, a basic text field will be created
    SunnyFormFieldBuilder<T?>? builder,
    bool? isReadOnly,
    bool captureNestedPaths = true,
    bool? enabled,
    AttributeAccepter<T?>? onReadOnlyTapped,
    FieldConverter<T?>? converter,
  }) : super(
          key: key,
          label: label,
          placeholder: placeholder,
          attribute: attribute,
          builder: builder,
          isReadOnly: isReadOnly,
          form: form,
          isRequired: isRequired,
          onChange: onChange,
          enabled: enabled,
          decoration: decoration,
          captureNestedPaths: captureNestedPaths,
          onReadOnlyTapped: onReadOnlyTapped,
          initialValue: form.get(JsonPath.of(attribute)),
//          initialValue: initialValue ??
//              converter?.call(controller.get(JsonPath.of(attribute))) ??
//              controller.get(JsonPath.of(attribute)),
          converter: converter,
        );

  AttributeFormField.forInitialValueGetter({
    Key? key,
    dynamic label,
    dynamic placeholder,
    dynamic attribute,
    InputDecoration? decoration,
    FormController? form,
    Getter<T>? initialValue,
    FormValueChanged<T?>? onChange,
    required bool isRequired,

    /// This builder should only build the raw input, without any extra stuff, like errors, help text,
    /// labels, or otherwise.  By default, a basic text field will be created
    SunnyFormFieldBuilder<T?>? builder,
    bool? isReadOnly,
    bool captureNestedPaths = true,
    bool? enabled,
    AttributeAccepter<T?>? onReadOnlyTapped,
    FieldConverter<T?>? converter,
    FocusNode? focusNode,
  }) : super(
          key: key,
          label: label,
          placeholder: placeholder,
          attribute: attribute,
          builder: builder,
          isRequired: isRequired,
          isReadOnly: isReadOnly,
          form: form,
          enabled: enabled,
          decoration: decoration,
          captureNestedPaths: captureNestedPaths,
          onReadOnlyTapped: onReadOnlyTapped,
          initialValue: initialValue?.call(),
          converter: converter,
          focusNode: focusNode,
          onChange: onChange,
        );
}
