import 'dart:async';

import 'package:flutter/cupertino.dart';
import 'package:sunny_core_widgets/sunny_core_widgets.dart';
import 'package:sunny_dart/sunny_dart.dart';
import 'package:sunny_essentials/state.dart';
import 'package:sunny_forms/builders/cross_fade.dart';

typedef StreamListBuilder = Widget Function(
    BuildContext context, List newElement);

/// Given a list of futures, will invoke the build once for each future, until all are resolved
class Futures extends StatefulWidget {
  final List<FutureOr> futures;
  final StreamListBuilder builder;
  final Widget? placeholder;
  final String name;

  Futures({
    required this.builder,
    required this.name,
    required this.futures,
    this.placeholder,
  }) : super(key: Key("futures-$name"));

  @override
  _Futures createState() => _Futures();
}

class _Futures extends BaseState<Futures> {
  @override
  Widget build(BuildContext context) {
    return StreamBuilder<List>(
      builder: (BuildContext context, AsyncSnapshot<List> snapshot) {
        return crossFade(
          first: widget.placeholder ?? emptyBox,
          second: widget.builder(context, snapshot.data ?? []),
          showFirst: snapshot.hasData,
        );
      },
      initialData: _futures.result,
      key: Key("futures-stream-${widget.name}"),
      stream: _futures.stream,
    );
  }

  late FuturesStream _futures;

  @override
  void initState() {
    super.initState();

    _futures = FuturesStream(widget.futures);
  }

  @override
  void dispose() {
    _futures.close();
    super.dispose();
  }
}
