import 'package:flutter/widgets.dart';
import 'package:info_x/sunny_get.dart';
import 'package:sunny_core_widgets/sunny_core_widgets.dart';
import 'package:sunny_forms/builders/focused.dart';
import 'package:sunny_forms/raw_text_field.dart';
import 'package:sunny_forms/sunny_form_controller.dart';
import 'package:sunny_forms/sunny_form_field.dart';
import 'package:sunny_service_stubs/services/named_icons.dart';

/// This widget will clear the contents of a form control (text box)
class ClearInputIcon<T> extends StatelessWidget {
  final TextEditingController? textEditingController;
  final FormValueChanged<T?>? notifier;

  final Widget? icon;
  final Widget? expandIcon;
  final FocusNode? focusNode;
  final Widget? unfocused;
  final bool showExpand;

  ClearInputIcon({
    Key? key,
    this.notifier,
    Widget? iconOverride,
    this.textEditingController,
    required this.focusNode,
    this.unfocused,
    required this.showExpand,
    this.expandIcon,
  })  : icon = iconOverride?.reIcon(color: sunnyColors.textLight, size: 20),
        super(key: key);

  @override
  Widget build(BuildContext context) {
    var _icon = icon ??
        sunny
            .get<NamedIconContainer>(context: context)
            .clear
            .build(color: sunnyColors.textLight, size: 30);
    return focusNode.showWhenFocused(
      Tappable(
          child: _icon,
          onTap: (context) async {
            if (textEditingController?.text == "") {
              focusNode?.unfocus();
            } else {
              notifier?.call(null, AttributeSource.control);
              textEditingController?.text = "";
            }
          }),
      unfocused: unfocused ??
          ((showExpand == true)
              ? Tappable(
                  child: expandIcon!,
                  onTap: (context) async {
                    FocusScope.of(context).requestFocus(focusNode);
                  })
              : null),
    );
  }
}
