import 'package:equatable/equatable.dart';
import 'package:flexidate/flexidate.dart';
import 'package:flutter/material.dart';
import 'package:sunny_platform_widgets/sunny_platform_widgets.dart';
import 'package:flutter_typeahead/flutter_typeahead.dart';
import 'package:provider/provider.dart';
import 'package:sunny_dart/helpers.dart';
import 'package:sunny_dart/sunny_get.dart';
import 'package:sunny_dart/typedefs.dart';
import 'package:sunny_forms/clear_input.dart';
import 'package:sunny_forms/form_fields/typeaheads.dart';
import 'package:sunny_forms/raw_text_field.dart';
import 'package:sunny_forms/smart_dates/smart_date_query.dart';
import 'package:sunny_forms/smart_dates/smart_date_service.dart';
import 'package:sunny_forms/sunny_fake_field_container.dart';
import 'package:sunny_forms/sunny_form_controller.dart';
import 'package:sunny_forms/sunny_form_field.dart';
import 'package:sunny_forms/sunny_form_theme.dart';
import 'package:sunny_forms/text_widgets.dart';
import 'package:sunny_forms/widget_extensions.dart';
import 'package:sunny_sdk_core/model.dart';
import 'package:sunny_sdk_core/services.dart';
import 'package:sunny_sdk_core/services/sunny.dart';
import 'package:sunny_service_stubs/sunny_service_stubs.dart';

class DatePickerControl extends StatefulWidget {
  final FormValueChanged<DateTime?>? onChange;
  final DateTime? initialValue;

  final Formatter<DateTime> formatter;

  /// If the text of the selection is different than the formatted date
  final String? initialSelection;
  final String? placeholder;
  final String? label;
  final bool? autofocus;
  final bool? enabled;
  final FocusNode? focusNode;
  final bool? adaptive;

  const DatePickerControl({
    Key? key,
    this.onChange,
    this.initialValue,
    this.placeholder,
    this.adaptive,
    required this.formatter,
    this.autofocus = false,
    this.label,
    this.enabled = true,
    this.focusNode,
    this.initialSelection,
  }) : super(key: key);

  /// Convenience for the different form field constructors
  static DatePickerControl ofFormField<T extends Object>({
    String? placeholder,
    required SunnyFormFieldState<T> state,
    required Mapping<T?, DateTime?> toResult,
    required Mapping<DateTime?, T?> fromResult,
    required Formatter<T?> formatter,
    bool? autofocus,
    bool? adaptive,
    TextInputAction? textInputAction,
  }) {
    return DatePickerControl(
        onChange: ((value, from) => state.updateValue(fromResult(value), from)),
        initialValue: toResult(state.value),
        initialSelection: formatter(state.value),
        autofocus: autofocus,
        adaptive: adaptive,
        formatter: (dateTime) => (formatter(fromResult(dateTime))),
        focusNode: state.focusNode,
        key: Key("dateTypeahead-${state.attribute.toKey()}"));
  }

  factory DatePickerControl.ofDateTimeField({
    required SunnyFormFieldState<DateTime> state,
    String? placeholder,
    bool autofocus = false,
    TextInputAction? textInputAction,
    bool dateOnly = false,
    bool useToday = true,
  }) =>
      DatePickerControl.ofFormField<DateTime>(
          state: state,
          autofocus: autofocus,
          fromResult: (result) => result,
          // placeholder: placeholder ?? "e.g. April 6, 2013",
          textInputAction: textInputAction,
          toResult: (dateTime) => dateTime,
          formatter: ((dateTime) =>
              smartDateFormat(
                dateTime,
                useToday: useToday,
                formatTime: !dateOnly,
              ) ??
              ''));

  @override
  _DatePickerControlState createState() => _DatePickerControlState();
}

class _DatePickerControlState extends State<DatePickerControl>
    with LoggingMixin {
  String? id;

  DateTime? value;
  String? label;

  @override
  void initState() {
    super.initState();
    id = uuid();
    label = widget.initialSelection;
    value = widget.initialValue;

    label ??= "";
  }

  @override
  dispose() {
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return FakeFieldContainer(
      onTap: () async {
        final pickedDate = await showPlatformDatePicker(
          context: context,
          initialDate: value ?? DateTime.now(),
          firstDate: DateTime.now().minusTimeSpan(TimeSpan(months: 6)),
          lastDate: DateTime.now().plusTimeSpan(TimeSpan(months: 1)),
        );
        if (pickedDate != null) {
          widget.onChange?.call(pickedDate, AttributeSource.control);
          setState(() {
            value = pickedDate;
            label = widget.formatter(pickedDate);
          });
        }
      },
      placeholder: widget.placeholder ?? 'Choose a date',
      icon: context.icons.calendar,
      focused: false,
      value: Container(child: Text(label ?? '', style: sunnyText.input1)),
    );
    // final input = inputWithIcon(
    //   prefix: context.icons.calendar.icon,
    //   suffix: ClearInputIcon(
    //     key: Key("clearInput-$id"),
    //     focusNode: widget.focusNode,
    //     notifier: widget.onChange,
    //     showExpand: false,
    //   ),
    //   color: widget.focusNode?.hasFocus == true || widget.initialValue != null
    //       ? sunnyColors.primaryColor
    //       : sunnyColors.textLight,
    //   input: Container(
    //       child: Text(label ?? widget.placeholder ?? 'Choose a date')),
    // );
    //
    // return widget.focusNode != null
    //     ? FocusTrapArea(
    //         focusNode: widget.focusNode!,
    //         child: input,
    //       )
    //     : input;
  }
}
