import 'package:flutter/cupertino.dart';
import 'package:sunny_dart/sunny_dart.dart';
import 'package:sunny_forms/date_typeahead.dart';
import 'package:sunny_forms/form_fields/select_field.dart';
import 'package:sunny_forms/options/smart_date_options_handler.dart';
import 'package:sunny_forms/smart_dates/smart_date_query.dart';
import 'package:sunny_forms/sunny_form_field.dart';
import 'package:sunny_forms/widget_extensions.dart';
import 'package:sunny_service_stubs/options.dart';

class DateTypeaheadOptionsFormField extends StatelessWidget {
  final dynamic label;
  final dynamic placeholder;
  final JsonPath<SmartDateQueryResult> attribute;
  final bool isRequired;
  final SmartDateQueryResult initialValue;
  final FormValueChanged<SmartDateQueryResult?>? onChange;

  final SmartDateOptionsConfig inputContext;

  const DateTypeaheadOptionsFormField({
    Key? key,
    this.label,
    this.placeholder,
    required this.isRequired,
    required this.attribute,
    required this.initialValue,
    this.onChange,
    required this.inputContext,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    final handler = context.smartDateOptions.withInputContext(inputContext);
    return SunnyFormField<SmartDateQueryResult>(
      label: label,
      attribute: attribute,
      isRequired: isRequired,
      placeholder: placeholder,
      builder: (BuildContext context, SunnyFormFieldState<SmartDateQueryResult> state) {
        return OptionSelectorControl<SmartDateQueryResult, SmartDateQueryResult>(
          viewHandler: handler.typeaheadHandler,
          key: Key("field-${attribute.path}"),
          handler: handler,
          placeholder: placeholder?.toString(),
          isRequired: isRequired,
          prefixIcon: context.icons.calendar.icon,
          onChange: (option, source) {
            state.updateValue(option?.value, source);
            onChange?.call(option?.value, source);
          },
          onError: (error) {
            state.error = error;
          },
          initialValue: state.value,
          focusNode: state.focusNode,
          focusMode: inputContext.defaultDates.isNotNullOrEmpty ? TypeaheadFocusMode.showAll : TypeaheadFocusMode.none,
        );
      },
    );
  }
}
