import 'package:dartxx/dartxx.dart';
import 'package:flexidate/flexidate.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:sunny_platform_widgets/sunny_platform_widgets.dart';
import 'package:observable_collections/observable_collections.dart';
import 'package:sunny_core_widgets/sunny_core_widgets.dart';
import 'package:sunny_dart/typedefs.dart';
import 'package:sunny_forms/attribute_form_field.dart';
import 'package:sunny_forms/ints.dart';
import 'package:sunny_forms/platform/platform_bottom_sheet.dart';
import 'package:sunny_forms/sunny_colors.dart';
import 'package:sunny_forms/sunny_fake_field_container.dart';
import 'package:sunny_forms/sunny_form_controller.dart';
import 'package:sunny_forms/widget_extensions.dart';

typedef GetDuration = TimeSpan Function();
typedef SetDuration = void Function(TimeSpan span);

Future<TimeSpan?> showDurationPicker(BuildContext context,
    {Key? key, TimeSpan? initialValue}) async {
  final duration = SingleValue(initialValue);
  return await showPlatformBottomSheet<TimeSpan>(
      context: context,
      builder: (context) {
        final intl = SunnyIntl.of(context)!;
        return Column(
          children: [
            Expanded(child: Container()),
            Container(
              color: Colors.white,
              alignment: Alignment.centerRight,
              child: PlatformButton(
                  onPressed: () {
                    Navigator.pop(context, duration.value);
                  },
                  child: Text(
                    intl.doneLabel,
                    style: SunnyTextStyles.primaryActionText,
                  )),
            ),
            // ignore: missing_required_param
            DurationPickerControl(
              key: key,
              initialValue: initialValue,
              onChange: (value) {
                duration.update(value);
              },
            )
          ],
        );
      });
}

class DurationPickerControl extends StatefulWidget {
  final ValueChanged<TimeSpan>? onChange;
  final TimeSpan? initialValue;
  final JsonPath? basePath;

  DurationPickerControl(
      {Key? key, this.basePath, this.onChange, this.initialValue})
      : super(key: key ?? Key("durationPicker-${basePath?.toKey()}"));

  @override
  _DurationPickerControlState createState() => _DurationPickerControlState();
}

class _DurationPickerControlState extends State<DurationPickerControl> {
  int hours = 0;
  int minutes = 0;

  FixedExtentScrollController? _hoursScroller;
  FixedExtentScrollController? _minutesScroller;

  @override
  void initState() {
    super.initState();
    final d = widget.initialValue!.toDuration();

    hours = d.inHours;
    minutes = d.inMinutes - (hours * 60);

    _hoursScroller = FixedExtentScrollController(initialItem: hours);
    _minutesScroller = FixedExtentScrollController(
        initialItem: (minutes == 0 ? minutes : minutes ~/ 5));
  }

  TimeSpan get _duration {
    final d = TimeSpan(minutes: minutes, hours: hours);
    return d;
  }

  @override
  Widget build(BuildContext context) {
    final _onChange = widget.onChange ?? (_) {};
    return SizedBox(
      height: 250,
      child: Row(
        mainAxisSize: MainAxisSize.max,
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          Flexible(
            flex: 1,
            child: CupertinoPicker(
              scrollController: _hoursScroller,
              offAxisFraction: -1 * SunnyPickers.offAxisFraction,
              backgroundColor: SunnyPickers.backgroundColor,
              itemExtent: SunnyPickers.itemExtent,
              useMagnifier: SunnyPickers.useMagnifier,
              magnification: SunnyPickers.magnification,
              children: [
                ...rangeOf(0, 12).map((m) {
                  return Center(
                    child: Text(
                      "$m",
                      style: SunnyTextStyles.pickerText,
                    ),
                  );
                })
              ],
              onSelectedItemChanged: (int value) {
                hours = value;
                _onChange(_duration);
              },
            ),
          ),
          Flexible(
            flex: 1,
            child: CupertinoPicker(
              offAxisFraction: -1 * SunnyPickers.offAxisFraction,
              backgroundColor: SunnyPickers.backgroundColor,
              itemExtent: SunnyPickers.itemExtent,
              useMagnifier: SunnyPickers.useMagnifier,
              magnification: SunnyPickers.magnification,
              children: [
                Container(
                    padding: EdgeInsets.only(left: 10),
                    alignment: AlignmentDirectional.centerStart,
                    child: Text("Hours", style: SunnyTextStyles.pickerLabel))
              ],
              onSelectedItemChanged: (int value) {},
            ),
          ),
          Flexible(
            flex: 1,
            child: CupertinoPicker(
              scrollController: _minutesScroller,
              offAxisFraction: SunnyPickers.offAxisFraction,
              backgroundColor: SunnyPickers.backgroundColor,
              itemExtent: SunnyPickers.itemExtent,
              useMagnifier: SunnyPickers.useMagnifier,
              magnification: SunnyPickers.magnification,
              children: [
                ...rangeOf(0, 12).map((m) {
                  return Center(
                      child:
                          Text("${m * 5}", style: SunnyTextStyles.pickerText));
                }),
              ],
              onSelectedItemChanged: (int value) {
                minutes = value * 5;
                _onChange(_duration);
              },
            ),
          ),
          Flexible(
            flex: 1,
            child: CupertinoPicker(
              offAxisFraction: -1 * SunnyPickers.offAxisFraction,
              backgroundColor: SunnyPickers.backgroundColor,
              itemExtent: SunnyPickers.itemExtent,
              useMagnifier: SunnyPickers.useMagnifier,
              magnification: SunnyPickers.magnification,
              children: [
                Container(
                    padding: EdgeInsets.only(left: 10),
                    alignment: AlignmentDirectional.centerStart,
                    child: Text("Minutes", style: SunnyTextStyles.pickerLabel))
              ],
              onSelectedItemChanged: (int value) {},
            ),
          ),
        ],
      ),
    );
  }
}

class DurationFormField extends StatelessWidget {
  final Formatter<TimeSpan?>? formatter;

  final JsonPath durationAttribute;
  final ValueChanged<TimeSpan?>? onChanged;
  final Key? widgetKey;
  final String? label;
  final bool isRequired;

//  DurationFormField.ofAmountAndUnit(
//      {Key key,
//      @required JsonPath amountAttribute,
//      @required JsonPath unitAttribute,
//      this.widgetKey,
//      this.onChanged,
//      this.formatter,
//      this.label})
//      : getter = (() {
//          final amountInMinutes = controller.get(amountAttribute);
//          if (amountInMinutes == null) return null;
//          return TimeSpan(minutes: (amountInMinutes as num)?.toInt());
//        }),
//        setter = ((TimeSpan duration) {
//          controller.set(amountAttribute, duration.toDuration().inMinutes);
//          controller.put(unitAttribute, TimeUnit.minutes_.value);
//        }),
//        durationAttribute = amountAttribute,
//        super(key: key);

  DurationFormField.ofDuration({
    Key? key,
    required this.durationAttribute,
    required this.isRequired,
    this.widgetKey,
    this.formatter,
    this.onChanged,
    this.label,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    final formatter = this.formatter ?? ((span) => span?.format() ?? '');
    return Padding(
      padding: const EdgeInsets.only(bottom: 5),
      child: AttributeFormField<TimeSpan>(
        attribute: durationAttribute,
        isRequired: isRequired,
        label: (ctx) => label,
        builder: (context, state) {
          final TimeSpan? duration = state.value;
          return GestureDetector(
            onTap: () async {
              final TimeSpan? duration = state.value;
              final newDuration = await showDurationPicker(
                context,
                key: Key("duration-picker-${durationAttribute.toKey()}"),
                initialValue: duration,
              );
              if (newDuration != null) {
                state.updateValue(newDuration, AttributeSource.control);
                onChanged?.call(newDuration);
              }
            },
            child: FakeFieldContainer(
              focused: false,
              icon: context.icons.clock,
              placeholder: "Choose a duration",
              value: textOrNull(formatter(duration)),
            ),
          );
        },
        isReadOnly: false,
        placeholder: "",
        key: widgetKey ?? Key("form-field-$durationAttribute"),
      ),
    );
  }
}

/// Assumes the duration is in minutes
Widget formatDurationWidget(num durationAmount) {
  final duration = Duration(minutes: durationAmount.toInt());
  if (duration.inMinutes == 0) {
    return PlaceholderText("Click to set the duration");
  }
  final hours = duration.inHours;
  final minutes = duration.inMinutes - (60 * hours);
  return PrimaryText([
    if (hours > 0) "$hours hour${hours > 1 ? 's' : ''}",
    if (minutes > 0) "$minutes minute${minutes > 1 ? 's' : ''}",
  ].join(" "));
}

/// Assumes the duration is in minutes
String? formatDuration(Duration duration) {
  if (duration.inMinutes == 0) return null;
  final hours = duration.inHours;
  final minutes = duration.inMinutes - (60 * hours);
  return [
    if (hours > 0) "$hours hour${hours > 1 ? 's' : ''}",
    if (minutes > 0) "$minutes minute${minutes > 1 ? 's' : ''}",
  ].join(" ");
}
