import 'package:flexidate/flexidate.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:sunny_platform_widgets/sunny_platform_widgets.dart';
import 'package:sunny_core_widgets/intl/sunny_intl.dart';
import 'package:sunny_forms/ints.dart';
import 'package:sunny_forms/sunny_colors.dart';
import 'package:sunny_sdk_core/model/flexi_date.dart';

Future<FlexiDate?> showHistoricalDatePicker(BuildContext context,
    {FlexiDate? initialValue}) async {
  final picker = initialValue?.copy() ?? FlexiDateData.now();
  return await showCupertinoModalPopup<FlexiDate>(
      context: context,
      builder: (context) {
        return Column(
          children: [
            Expanded(child: Container()),
            Container(
              color: Colors.white,
              alignment: Alignment.centerRight,
              child: PlatformButton(
                  onPressed: () => Navigator.of(context).pop(picker),
                  child: Text(
                    SunnyIntl.of(context)!.doneLabel,
                    style: Theme.of(context).textTheme.bodyText2!.copyWith(
                        color: Theme.of(context).primaryColor,
                        fontWeight: FontWeight.w500),
                  )),
            ),
            HistoricalDatePicker(initialValue: picker)
          ],
        );
      });
}

class HistoricalDatePicker extends StatefulWidget {
  final ValueChanged<FlexiDate?>? onChange;
  final FlexiDate? initialValue;

  const HistoricalDatePicker({Key? key, this.onChange, this.initialValue})
      : super(key: key);

  @override
  _HistoricalDatePickerState createState() => _HistoricalDatePickerState();
}

class _HistoricalDatePickerState extends State<HistoricalDatePicker> {
  FlexiDate? picker;

  FixedExtentScrollController? _dayScroll;
  FixedExtentScrollController? _monthScroll;
  FixedExtentScrollController? _yearScroll;

  @override
  void initState() {
    super.initState();

    picker = widget.initialValue ?? FlexiDateData.now();
    _monthScroll =
        FixedExtentScrollController(initialItem: (picker?.month ?? 1) - 1);
    _dayScroll =
        FixedExtentScrollController(initialItem: (picker?.day ?? 1) - 1);
    _yearScroll = FixedExtentScrollController(
        initialItem: picker?.year == null ? 0 : picker!.year! - 1919);
  }

  notifyListener() {
    widget.onChange?.call(picker);
  }

  @override
  void dispose() {
    _dayScroll!.dispose();
    _monthScroll!.dispose();
    _yearScroll!.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      height: 250,
      child: Row(
        mainAxisSize: MainAxisSize.max,
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          CupertinoPicker(
            scrollController: _monthScroll,
            offAxisFraction: -1 * SunnyPickers.offAxisFraction,
            backgroundColor: SunnyPickers.backgroundColor,
            itemExtent: SunnyPickers.itemExtent,
            useMagnifier: SunnyPickers.useMagnifier,
            magnification: SunnyPickers.magnification,
            children: months.map((m) {
              return Center(
                  child: Text(
                SunnyIntl.of(context)!.month(m),
                style: SunnyTextStyles.pickerText,
              ));
            }).toList(),
            onSelectedItemChanged: (int value) {
              (picker as FlexiDateData).month = value + 1;
              notifyListener();
            },
          ),
          CupertinoPicker(
            scrollController: _dayScroll,
            offAxisFraction: SunnyPickers.offAxisFraction,
            backgroundColor: SunnyPickers.backgroundColor,
            itemExtent: SunnyPickers.itemExtent,
            useMagnifier: SunnyPickers.useMagnifier,
            magnification: SunnyPickers.magnification,
            children: rangeOf(1, 31).map((m) {
              return Center(
                child: Text(
                  "$m",
                  style: SunnyTextStyles.pickerText,
                ),
              );
            }).toList(),
            onSelectedItemChanged: (int value) {
              (picker as FlexiDateData).day = value + 1;
              notifyListener();
            },
          ),
          CupertinoPicker(
            scrollController: _yearScroll,
            offAxisFraction: SunnyPickers.offAxisFraction,
            backgroundColor: SunnyPickers.backgroundColor,
            itemExtent: SunnyPickers.itemExtent,
            useMagnifier: SunnyPickers.useMagnifier,
            magnification: SunnyPickers.magnification,
            children: <int?>[null]
                .followedBy(rangeOf(1920, DateTime.now().year))
                .map((m) {
              return Center(
                child: Text(
                  "${m ?? '----'}",
                  style: SunnyTextStyles.pickerText,
                ),
              );
            }).toList(),
            onSelectedItemChanged: (int value) {
              (picker as FlexiDateData).year = value == 0 ? null : 1919 + value;
              notifyListener();
            },
          ),
        ]
            .map((item) => Flexible(
                  flex: 1,
                  child: item,
                ))
            .toList(),
      ),
    );
  }
}
