import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:google_maps_webservice/places.dart';
import 'package:logging/logging.dart';
import 'package:sunny_core_widgets/sunny_core_widgets.dart';
import 'package:sunny_forms/builders/future.dart';
import 'package:sunny_forms/platform/platform_flat_button.dart';
import 'package:sunny_sdk_core/model/sunny_place.dart';

import 'location_utils.dart';

// ignore: unused_element
final _log = Logger("locationPicker");

/// See [SunnyLocation] widget
class LocationTile extends StatelessWidget {
  final SunnyPlace? place;
  final Widget? emptyState;
  final bool showOpenInMapsLink;

  LocationTile({
    Key? key,
    required this.place,
    this.emptyState,
    this.showOpenInMapsLink = true,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return future(context,
        waitFor: lookupPlaceInGoogle(place?.googlePlaceId),
        allowNull: true, builder: (context, PlaceDetails? placeDetails) {
      final lines = addressToString(place) ?? [];
      return Expanded(
        child: Row(
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: [
            Container(
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                mainAxisSize: MainAxisSize.min,
                children: [
                  if (lines.isEmpty && emptyState != null) emptyState!,
                  for (final line in lines) PrimaryText(line, nowrap: true),
                ],
              ),
            ),
//          if (placeDetails?.photos?.isNotEmpty == true)
//            Image(
//              image: CachedNetworkImageProvider(_photoUrl(placeDetails?.photos?.first?.photoReference)),
//              frameBuilder: (context, child, _, __) => ClipOval(child: child),
//            ),
            horizontalSpace,
            if (placeDetails != null && showOpenInMapsLink)
              Container(
                child: PlatformFlatButton(
                  padding: EdgeInsets.all(10),
                  child: Text("Open in Maps"),
                  onPressed: () => openMap(context, placeDetails),
                ),
              )
          ],
        ),
      );
    });
  }
}
