import 'dart:async';

import 'package:extended_image/extended_image.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:image_cropper/image_cropper.dart';
import 'package:logging/logging.dart';
import 'package:multi_image_picker/multi_image_picker.dart';
import 'package:path_provider/path_provider.dart';
import 'package:permission_handler/permission_handler.dart';
import 'package:pfile_picker/pfile_picker.dart';
import 'package:sunny_core_widgets/routes/modals.dart';
import 'package:sunny_dart/sunny_dart.dart' as p;
import 'package:sunny_dart/sunny_dart.dart';
import 'package:sunny_forms/image/crop_controls.dart';
import 'package:sunny_forms/image/cropper.dart';
import 'package:sunny_forms/media/file_source_type.dart';
import 'package:sunny_forms/options.dart';
import 'package:sunny_forms/permissions.dart';
import 'package:sunny_forms/sunny_hud.dart';
import 'package:sunny_sdk_core/model/media_service.dart';
import 'package:sunny_service_stubs/services/media_types.dart';

final _log = Logger("imagePicker");

/// This pops the file picker/cropper and returns a single file.
/// @deprecated
Future<PFile?> openImagePicker(
  BuildContext context, {
  FileChanged? onChange,
  FilesUploaded? onSelected,
  bool crop = true,
  VoidCallback? onPermissionsInitialized,
  bool preemptPermissions = true,
  bool allowThirdParties = false,
  extraOptions,
}) async {
  try {
    // ignore: deprecated_member_use_from_same_package
    final imageFn = allowThirdParties ? filePickerFn : multiImageSelector;
    // ignore: deprecated_member_use_from_same_package
    final selection = await openImageListPicker(context,
        onChange: onChange,
        maxSelections: 1,
        crop: crop,
        onSelected: onSelected,
        onPermissionsInitialized: onPermissionsInitialized,
        extraOptions: extraOptions,
        imageSelector: imageFn,
        preemptPermission: preemptPermissions);
    return selection.orEmptyIter().firstOr();
  } catch (e, stack) {
    _log.severe("Image picker error: $e", e, stack);
    return null;
  }
}

@deprecated
typedef MediaSelectFn = Future<List<PFile?>> Function(BuildContext context,
    {MediaContentType? mediaType, int? maxSelections, dynamic extraOptions});

@deprecated
typedef CropFn = Future<PFile> Function(BuildContext context, PFile file);

@deprecated
Future<PFile?> cropFallback(BuildContext context, PFile source) async {
  try {
    final totalSize = MediaQuery.of(context).size;
    var bytes = source.bytes ?? await source.awaitData;
    // ignore: missing_required_param
    return await Modals.open<PFile>(
      context,
      expand: true,
      height: totalSize.height - 100,
      width: totalSize.width - 100,
      constraints: BoxConstraints(
        maxHeight: totalSize.height - 100,
        maxWidth: totalSize.width - 100,
      ),
      builder: (context) {
        return ExtendedImage.memory(
          bytes,
        );
      },
    );
  } catch (e, stack) {
    _log.severe("Error $e", e, stack);
    rethrow;
  }
}

@deprecated
Future<PFile?> cropperFallback(BuildContext context, PFile source) async {
  try {
    final bytes = await source.awaitData;
    // ignore: missing_required_param
    return await Modals.open(
      context,
      builder: (context) {
        return Cropper(image: bytes);
      },
    );
  } catch (e, stack) {
    _log.severe("Error $e", e, stack);
    rethrow;
  }
}

final imageCropper = ImageCropper();

@deprecated
Future<PFile?> cropNative(BuildContext context, PFile imageFile) async {
  final cropped = await imageCropper.cropImage(
    sourcePath: imageFile.path!,
    iosUiSettings: IOSUiSettings(
      showActivitySheetOnDone: false,
      hidesNavigationBar: true,
    ),
    aspectRatio: CropAspectRatio(ratioX: 1, ratioY: 1),
    maxWidth: 2048,
    maxHeight: 2048,
  );

  if (cropped == null) return null;
  return PFile.of(cropped);
}

// class FileSourceType extends Equatable {
//   final String label;
//   final String description;
//
//   const FileSourceType.files()
//       : label = "Select from files",
//         description = "iCloud, Dropbox, Device files, etc.";
//
//   const FileSourceType.images()
//       : label = "Select from gallery",
//         description = "Photos and videos on your phone";
//
//   @override
//   List<Object> get props => [label, description];
//
//   Option<FileSourceType> toOption() => Option(this, label: label);
// }

@deprecated
Future<List<PFile>> filePickerFn(BuildContext context,
    {int? maxSelections = 1, MediaContentType? mediaType, extraOptions}) async {
  List<PFile> res;
  if (infoX.isIOS) {
    final isIm =
        await showPicker(context, extraOptions: extraOptions, options: {
      const FileSourceType.files().toOption(),
      const FileSourceType.gallery().toOption(),
    });

    if (isIm == null) {
      return [];
    }

    if (isIm.key == const FileSourceType.files()) {
      final sel = await FilePicker.platform.pickFiles(
        // type: FileType.image,
        type: FileType.custom,
        allowCompression: true,
        // allowedExtensions: mediaType.acceptedFileTypes,
        allowMultiple: maxSelections! > 1,
        withReadStream: true,
        withData: true,
      );
      res = [...?sel?.files];
    } else if (isIm.key == const FileSourceType.gallery()) {
      final sel = await FilePicker.platform.pickFiles(
        type: mediaType == MediaTypes.videoExt
            ? FileType.video
            : mediaType == MediaTypes.imageExt
                ? FileType.image
                : FileType.any,
        allowCompression: true,
        allowMultiple: maxSelections! > 1,
        withReadStream: true,
        withData: true,
      );
      res = [...?sel?.files];
    } else {
      throw "Illegal state";
    }
  } else {
    final sel = await FilePicker.platform.pickFiles(
      // type: FileType.image,
      type: FileType.custom,
      allowCompression: true,
      // allowedExtensions: ["jpg", "png", "jpeg", "gif"],
      allowMultiple: maxSelections! > 1,
      withReadStream: true,
      withData: true,
    );
    res = [...?sel?.files];
  }
  return res;
}

@deprecated
Future<List<PFile?>> multiImageSelector(
  BuildContext context, {
  int? maxSelections = 1,
  MediaContentType? mediaType,
  dynamic extraOptions,
}) async {
  final media = await MultiImagePicker.pickImages(
      maxImages: maxSelections!,
      enableCamera: true,
      materialOptions: MaterialOptions(
        textOnNothingSelected: "Nothing was selected!",
        autoCloseOnSelectionLimit: true,
        lightStatusBar: true,
      ),
      cupertinoOptions: CupertinoOptions(
        autoCloseOnSelectionLimit: true,
      ));

  if (media.isNotEmpty != true) {
    return [];
  }

  const double max = 1800.0;
  final images = await Future.wait(media.map((asset) async {
    final w = asset.originalWidth!;
    final h = asset.originalHeight!;
    double ratio, ww, hh;
    if (asset.isLandscape) {
      if (w > max) {
        ratio = max / w;
      } else {
        ratio = 1;
      }
    } else {
      if (h > max) {
        ratio = max / h;
      } else {
        ratio = 1;
      }
    }
    ww = ratio * w;
    hh = ratio * h;
    final image = await asset.getThumbByteData(ww.truncate(), hh.truncate());
    await getTemporaryDirectory();
    final buffer = image.buffer;
    final bytes = buffer.asUint8List(image.offsetInBytes, image.lengthInBytes);
    return PFile.of(bytes, name: asset.name, size: bytes.length);
  }));

  return images;
}

// Future<List<PFile>> openMediaListPicker(
//   BuildContext context,
//   MediaContentType mediaType, {
//   VoidCallback onPermissionsInitialized,
//   FilesUploaded onSelected,
//   FileChanged onChange,
//   int maxSelections = 1,
//   bool crop = true,
//   bool preemptPermission = false,
//   MediaSelectFn imageSelector = filePickerFn,
// }) async {
//   final permission = await checkAndRequestPermissions(
//     context,
//     title: (intl) => intl.permissionRequestPhotos,
//     permission: Permission.photos,
//     preemptPermission: preemptPermission,
//   );
//   switch (permission) {
//     case SunnyPermissionStatus.rejected:
//       SunnyHud.error(context, "You haven't granted access to your photos.");
//       break;
//     case SunnyPermissionStatus.granted:
//       try {
//         onPermissionsInitialized?.call();
//         final files = await imageSelector(
//           context,
//           maxSelections: maxSelections,
//           mediaType: mediaType,
//         );
//         onSelected?.call(files);
//         return files;
//       } on NoImagesSelectedException {
//         return [];
//       }
//       break;
//     case SunnyPermissionStatus.later:
//       return [];
//       break;
//   }
//   return null;
// }

@deprecated
Future<List<PFile?>?> openImageListPicker(
  BuildContext context, {
  VoidCallback? onPermissionsInitialized,
  FilesUploaded? onSelected,
  FileChanged? onChange,
  int maxSelections = 1,
  bool crop = true,
  bool preemptPermission = true,
  MediaSelectFn imageSelector = multiImageSelector,
  dynamic extraOptions,
}) async {
  final permission = await checkAndRequestPermissions(
    context,
    title: (intl) => intl!.permissionRequestPhotos,
    permission: Permission.photos,
    preemptPermission: preemptPermission,
  );

  switch (permission) {
    case SunnyPermissionStatus.rejected:
      SunnyHud.error(context, "You haven't granted access to your photos.");
      break;
    case SunnyPermissionStatus.granted:
      try {
        onPermissionsInitialized?.call();
        final images = await imageSelector(context,
            maxSelections: maxSelections, extraOptions: extraOptions);
        onSelected?.call(images);
        if (crop == true && images.length == 1) {
          final imageFile = images.first;
          onChange?.call(imageFile, false);
          final cropPt = await ((infoX.isIOS || infoX.isAndroid)
              ? cropNative(context, imageFile!)
              : cropFallback(context, imageFile!));
          // Upload new image
          onChange?.call(cropPt, true);
          return [cropPt];
        } else {
          return images;
        }
      } on NoImagesSelectedException {
        return [];
      }
    case SunnyPermissionStatus.later:
      return [];
  }
  return null;
}

typedef FilesUploaded = void Function(List<PFile?> file);
typedef FileChanged = void Function(PFile? file, bool isFinal);
typedef ErrorCallback = void Function(Object error, StackTrace stack);

class ToPFile {}

// extension FileToPFileExt on File {
//   PFile toPFile() {
//     return PFile(
//         path: this.path,
//         name: JsonPath.parsed(path).last,
//         bytes: this.readAsBytesSync(),
//         size: this.lengthSync());
//   }
// }

extension PFileWriteExt on PFile {
  // Future<File> toFile() async {
  //   final imageData = await this.data;
  //   final tmpDir = await getTemporaryDirectory();
  //   final tmpFile = File("${tmpDir.path}/${name ?? uuid()}");
  //   if (tmpFile.existsSync()) {
  //     return tmpFile;
  //   } else {
  //     tmpFile.writeAsBytesSync(imageData, flush: true);
  //     return tmpFile;
  //   }
  // }

  // PFile withNewData(Uint8List data) {
  //   return PFile(
  //     name: name,
  //     path: path,
  //     bytes: data,
  //     size: data.length,
  //   );
  // }

  // FutureOr<Uint8List> get data {
  //   if (this.bytes != null) {
  //     return bytes;
  //   } else if (readStream != null) {
  //     return readStream
  //         .toList()
  //         .then((input) => input.expand((element) => element).toList())
  //         .then((flattened) => Uint8List.fromList(flattened));
  //   } else {
  //     return Uint8List.fromList([]);
  //   }
  // }

  String? get extension {
    String? np = this.name ?? path;
    return np?.extension;
  }

  // Stream<List<int>> get dataStream {
  //   if (this.bytes != null) {
  //     return Stream.fromIterable([bytes]);
  //   } else if (readStream != null) {
  //     return readStream;
  //   } else {
  //     return Stream.empty();
  //   }
  // }
}
