import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:meta/meta.dart'; // ignore: unused_import, directives_ordering
import 'package:sunny_core_widgets/sunny_core_widgets.dart';
import 'package:sunny_dart/sunny_dart.dart';
import 'package:sunny_forms/input_parsing.dart';
import 'package:sunny_forms/raw_text_field.dart';
import 'package:sunny_forms/sunny_form_controller.dart';
import 'package:sunny_forms/sunny_form_field.dart';
import 'package:sunny_sdk_core/sunny_sdk_core.dart';
import 'package:sunny_service_stubs/models.dart';

class RangeControl extends StatefulWidget {
  final FormValueChanged<IRange> onChange;
  final Consumer<ValidationError>? onError;
  final IRange? initialValue;
  final JsonPath basePath;

  /// Will apply to both range inputs
  final Icon? prefixIcon;
  final double? width;

  const RangeControl({
    Key? key,
    this.prefixIcon,
    required this.basePath,
    required this.onChange,
    required this.initialValue,
    this.onError,
    this.width,
  }) : super(key: key);

  @override
  _RangeControlState createState() => _RangeControlState();
}

class _RangeControlState extends State<RangeControl> {
  double? _start;
  double? _end;

  @override
  void initState() {
    super.initState();
    _start = widget.initialValue?.start?.toDouble();
    _end = widget.initialValue?.end?.toDouble();
  }

  _onChange(AttributeSource source) {
    widget.onChange.call(IRange.of(start: _start, end: _end), source);
  }

  @override
  Widget build(BuildContext context) {
    return row([
      Container(
        width: 60,
        child: TextControl<double?>.formField(
          attribute: widget.basePath.plus(JsonPath.of("start")),
          prefix: widget.prefixIcon,
          converter: InputConverters.doubles,
          onChange: (start, source) {
            _start = start;
            _onChange(source);
          },
          onError: (error) {
            widget.onError?.call(error);
          },
        ),
      ),
      hpad(Text(
        " - ",
        textAlign: TextAlign.center,
        style: TextStyle(fontWeight: FontWeight.bold, fontSize: 20),
      )),
      Container(
        width: 60,
        child: TextControl<double?>.formField(
          attribute: widget.basePath.plus(JsonPath.of("end")),
          prefix: widget.prefixIcon,
          converter: InputConverters.doubles,
          onChange: (end, source) {
            _end = end;
            _onChange(source);
          },
          onError: (error) {
            widget.onError?.call(error);
          },
        ),
      ),
    ]);
  }
}
