import 'package:dartxx/dartxx.dart';
import 'package:flexidate/flexidate.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:sunny_dart/typedefs.dart';
import 'package:sunny_essentials/provided.dart';
import 'package:sunny_essentials/state.dart';
import 'package:sunny_forms/sunny_form_controller.dart';
import 'package:sunny_forms/sunny_form_field.dart';
import 'package:sunny_forms/time_span_typeahead.dart';

class TimeSpanTypeaheadFormField extends StatefulWidget {
  final dynamic label;
  final dynamic placeholder;
  final Formatter<TimeSpan?>? formatter;
  final JsonPath? attribute;
  final bool isRequired;
  final TimeSpan? initialValue;
  final FormController? controller;
  final ValueChanged<TimeSpan?>? onChange;
  final bool? showIcon;

  const TimeSpanTypeaheadFormField({
    Key? key,
    this.label,
    this.placeholder,
    required this.isRequired,
    this.attribute,
    this.initialValue,
    this.onChange,
    this.controller,
    this.formatter,
    this.showIcon,
  }) : super(key: key);

  @override
  _TimeSpanTypeaheadFormFieldState createState() =>
      _TimeSpanTypeaheadFormFieldState();
}

class _TimeSpanTypeaheadFormFieldState
    extends BaseState<TimeSpanTypeaheadFormField> {
  TextEditingController? _tc;
  Formatter<TimeSpan?>? _formatter;
  late FormController _form;
  @override
  void initState() {
    super.initState();
    _formatter = widget.formatter ??
        (timespan) {
          if (timespan == null) return '';
          return timespan.format();
        };

    _form = widget.controller ??
        Provided.find<SunnyFormController>(context) ??
        Provided.get<FormController>(context);

    _tc =
        streamText(_form.stream.get(widget.attribute).control.map((dynamic v) {
      return _formatter!(v as TimeSpan?);
    }));

    _tc!.text = _formatter!(_form.get(widget.attribute!));
  }

  @override
  Widget build(BuildContext context) {
    return SunnyFormField<TimeSpan>(
      label: widget.label,
      isRequired: widget.isRequired,
      attribute: widget.attribute,
      form: _form,
      placeholder: widget.placeholder,
      builder: (BuildContext context, state) {
        return TimeSpanTypeaheadControl(
          key: Key("field-${widget.attribute!.path}"),
          formatter: _formatter,
          showIcon: widget.showIcon,
          textController: _tc,
          placeholder: widget.placeholder?.toString(),
          onChange: (suggestion, source) {
            widget.onChange?.call(suggestion);
            state.updateValue(suggestion, source);
          },
          focusNode: state.focusNode,
          initialValue: state.value,
        );
      },
    );
  }
}
