import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:sunny_platform_widgets/sunny_platform_widgets.dart';
import 'package:sunny_forms/dividing_line.dart';
import 'package:sunny_forms/platform/platform_stateful_widget_base.dart';
import 'package:sunny_forms/sunny_form_submit.dart';
import 'package:sunny_service_stubs/services.dart';

class HeaderButton
    // ignore: deprecated_member_use
    extends PlatformStatefulWidgetBase<PlatformButton, FlatButton> {
  final String? text;
  final NamedIcon? icon;
  final TextStyle? style;
  final FutureCallback? onTap;
  final bool? isPrimary;
  final bool? isLight;
  final EdgeInsets? padding;
  final EdgeInsets? margin;

  HeaderButton(
    this.text, {
    Key? key,
    this.onTap,
    this.padding,
    this.margin,
    this.isLight,
    this.style,
    this.isPrimary,
  })  : icon = null,
        super(key: key ?? Key("header-button-$text"));

  HeaderButton.icon(
    this.icon, {
    Key? key,
    FutureCallback? onTap,
    this.padding,
    this.margin,
    this.isLight,
    this.style,
    this.isPrimary,
  })  : text = null,
        onTap = onTap ?? comingSoonCallback,
        super(key: key ?? Key("header-button-$icon"));

  HeaderButton.save(FutureCallback? onTap,
      {this.text = "Save", this.margin, this.icon})
      : onTap = onTap ?? comingSoonCallback,
        isPrimary = true,
        style = null,
        isLight = false,
        padding = null,
        super(key: Key("header-button-save"));

  HeaderButton.cancel({
    String? text,
    this.icon,
    Key key = const Key("cancel-header-button"),
    this.padding,
    this.margin,
    this.isLight,
    this.isPrimary = false,
    this.style,
  })  : text = text ?? "Cancel",
        onTap = ((context) {
          Navigator.pop(context);
        }),
        super(key: key);

  @override
  State createState() => HeaderButtonState();
}

class HeaderButtonState
    extends PlatformWidgetState<Widget, Widget, HeaderButton> {
  bool _isSubmitting = false;

  @override
  void initState() {
    super.initState();
    _isSubmitting = false;
  }

  TextStyle get buttonTextStyle {
    final baseStyle = widget.style ??
        (widget.isPrimary == true
            ? sunnyText.body1Link
            : sunnyText.body1Normal);
    if (_isSubmitting) {
      return baseStyle.copyWith(color: baseStyle.color!.withOpacity(0.5));
    }
    return baseStyle;
  }

  EdgeInsets get margin {
    return widget.margin ?? sunnySpacing.horiz;
  }

  @override
  Widget createCupertinoWidget(BuildContext context) {
    return CupertinoButton(
      // padding: widget.padding ?? sunnySpacing.horiz,
      onPressed: _submit,
      child: Text(
        widget.text!,
        maxLines: 1,
        softWrap: false,
        overflow: TextOverflow.visible,
        style: buttonTextStyle,
      ),
    );
  }

  @override
  Widget createMaterialWidget(BuildContext context) {
    if (widget.icon != null) {
      return IconButton(
        color: sunnyColors.primaryColor,
        icon: widget.icon!.build(color: sunnyColors.primaryColor, size: 24),
        onPressed: _submit,
      );
    } else {
      // ignore: deprecated_member_use
      return FlatButton(
        // colorBrightness: Brightness.dark,
        onPressed: _submit,
        child: Text(widget.text!, style: buttonTextStyle),
      );
    }
  }

  VoidCallback? get _submit => _isSubmitting
      ? null
      : (() async {
          if (_isSubmitting) return;
          setState(() {
            _isSubmitting = true;
          });
          await widget.onTap!(context);
          setState(() {
            _isSubmitting = false;
          });
        });
}

void comingSoon(BuildContext context, [String? message]) {
  comingSoonCallback(context, message)();
}

comingSoonCallback(BuildContext context, [String? message]) {
  return (_) async {};
}
