import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:sunny_dart/sunny_dart.dart';
import 'package:sunny_dart/ui.dart';

abstract class IconInfo {
  IconType get type;

  set type(IconType type);

  String get icon;

  set icon(String icon);

  String get background;

  set background(String background);

  bool get circular;

  set circular(bool circular);

  Color? get backgroundColor => background.toColor();
}

class IconType extends IconTypeBase {
  const IconType._internal(String value) : super._internal(value);

  const IconType.unknown(String value) : super.unknown(value);

  // ignore: non_constant_identifier_names
  static const IconType image_ = IconType._internal("IMAGE");

  // ignore: non_constant_identifier_names
  static const IconType named_ = IconType._internal("NAMED");

  static IconType? fromJson(dynamic data) {
    if (data == null) return null;
    switch ("$data") {
      case "IMAGE":
        return IconType.image_;
      case "NAMED":
        return IconType.named_;
      default:
        return IconType.unknown("$data");
    }
  }

  static List<IconType> values = [IconType.image_, IconType.named_];

  static dynamic encode(IconType data) => data.value;

  dynamic toJson() => json.encode(value);
}

class IconTypeBase extends MLiteral<String> {
  final bool isKnown;

  const IconTypeBase._internal(String value)
      : isKnown = true,
        super(value);

  const IconTypeBase.unknown(String value)
      : isKnown = false,
        super(value);
}
