import 'dart:typed_data';
import 'dart:ui';

import 'package:equatable/equatable.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/widgets.dart';
import 'package:pfile_picker/pfile_picker.dart';
import 'package:sunny_dart/helpers.dart';
import 'package:sunny_dart/helpers/strings.dart';

/// Reads a stream of bytes as an image -
// ignore: must_be_immutable
class StreamImageProvider extends ImageProvider<StreamImageProvider>
    implements Equatable {
  final String name;
  final int size;
  final Stream<List<int>> _source;
  Future<Uint8List>? _read;

  StreamImageProvider(
      {required this.name,
      required this.size,
      required Stream<List<int>> source})
      : _source = source;

  StreamImageProvider.ofPFile(PFile file)
      : _source = file.openStream(),
        name = file.name ?? uuid(),
        size = file.size;

  StreamImageProvider.ofStream({required Stream<List<int>> source})
      : _source = source,
        size = -1,
        name = uuid();

  @override
  List<Object> get props => [name, size];

  @override
  bool get stringify => true;

  static Future<Codec> _loadAsync(
      StreamImageProvider provider, DecoderCallback decode) async {
    provider._read ??= provider._readBytes();
    final bytes = await provider._read!;
    return decode(bytes);
  }

  Future<Uint8List> _readBytes() async {
    // var bytes = Uint8List(size);
    // var i = 0;
    return Uint8List.fromList(
        (await _source.toList()).expand((element) => element).toList());
    // await for (var chunk in _source) {
    //   for (var byte in chunk) {
    //     bytes[i++] = byte;
    //   }
    // }
    // return bytes;
  }

  @override
  ImageStreamCompleter load(StreamImageProvider key, DecoderCallback decode) {
    return MultiFrameImageStreamCompleter(
      codec: _loadAsync(key, decode),
      debugLabel: key.toString(),
      scale: 1.0,
    );
  }

  @override
  Future<StreamImageProvider> obtainKey(ImageConfiguration configuration) {
    return SynchronousFuture<StreamImageProvider>(this);
  }
}
