import 'package:flutter/widgets.dart';
import 'package:image_cropper/image_cropper.dart';
import 'package:logging/logging.dart';
import 'package:pfile_picker/pfile_picker.dart';
import 'package:sunny_core_widgets/routes/modals.dart';
import 'package:sunny_forms/image/crop_controls.dart';

final _log = Logger("croppers");

typedef CropFn = Future<PlatformFile> Function(
    BuildContext context, PlatformFile file);

class Croppers {
  static final _imageCropper = ImageCropper();
  static Future<PFile?> cropFallback(BuildContext context, PFile source,
      {double? cropAspectRatio}) async {
    try {
      final totalSize = MediaQuery.of(context).size;
      var bytes = source.bytes ?? await source.awaitData;
      final height = totalSize.height - 100;
      final width = totalSize.width - 100;
      // ignore: missing_required_param
      return await Modals.open<PFile>(
        context,
        expand: true,
        constraints: BoxConstraints(
          maxHeight: height - 100,
          maxWidth: width - 100,
        ),
        builder: (context) {
          return CropImagePage.ofPlatformFile(
            source: source,
            bytes: bytes,
          );
        },
      );
    } catch (e, stack) {
      _log.severe("Error $e", e, stack);
      rethrow;
    }
  }

  static Future<PFile?> cropperFallback(
      BuildContext context, PFile source) async {
    try {
      // ignore: missing_required_param
      final totalSize = MediaQuery.of(context).size;
      var bytes = source.bytes ?? await source.awaitData;
      // ignore: missing_required_param
      return await Modals.open<PFile>(
        context,
        expand: true,
        height: totalSize.height - 100,
        width: totalSize.width - 100,
        builder: (context) {
          return CropImagePage(
              sourceName: source.name ?? 'unnamed_file', source: bytes);
        },
      );
    } catch (e, stack) {
      _log.severe("Error $e", e, stack);
      rethrow;
    }
  }

  static Future<PFile?> cropNative(BuildContext context, PFile imageFile,
      {double? cropAspectRatio}) async {
    final cropped = await _imageCropper.cropImage(
      sourcePath: imageFile.path!,
      iosUiSettings: IOSUiSettings(
        showActivitySheetOnDone: false,
        hidesNavigationBar: true,
      ),
      aspectRatio: CropAspectRatio(ratioX: cropAspectRatio ?? 1.0, ratioY: 1.0),
      maxWidth: 2048,
      maxHeight: 2048,
    );

    if (cropped == null) return null;
    return PFile.of(cropped, name: imageFile.name);
  }
}
