import 'dart:async';

import 'package:flutter/material.dart';
import 'package:meta_forms/mverse.dart';
import 'package:sunny_dart/sunny_dart.dart' as p;
import 'package:sunny_dart/sunny_dart.dart';
import 'package:sunny_forms/form_fields/time_span_field.dart';
import 'package:sunny_forms/raw_text_field.dart';
import 'package:sunny_forms/sunny_form_controller.dart';
import 'package:sunny_forms/sunny_forms.dart';
import 'package:sunny_service_stubs/models.dart';

Future<F?> showMediaUrlForm<F extends Object>(BuildContext context,
    {FlutterMediaContentType<F>? mediaType}) {
  Widget? _w;
  return Modals.open<F>(
    context,
    displayDragHandle: true,
    builder: (context) {
      return _w ??= MediaUrlForm<F>(mediaType: mediaType);
    },
    settings: null,
    expand: false,
  );
}

class MediaUrlForm<F extends Object> extends StatefulWidget {
  final FlutterMediaContentType<F>? mediaType;

  const MediaUrlForm({Key? key, this.mediaType}) : super(key: key);

  @override
  _MediaUrlFormState<F> createState() => _MediaUrlFormState<F>();
}

class _MediaUrlFormState<F extends Object> extends State<MediaUrlForm<F>>
    with LoggingMixin {
  FlutterMediaContentType<F>? get mediaType => widget.mediaType;

  F? fact;

  @override
  void initState() {
    super.initState();
    fact = mediaType!.newContent();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      padding: EdgeInsets.only(bottom: 40),
      child: Container(
        decoration: BoxDecoration(
            borderRadius: SunnySpacing.twelvePxRadiusAll,
            color: sunnyColors.scaffoldBackground),
        child: column(
          [
            column([
              Center(
                  child: AppBar(
                      backgroundColor: sunnyColors.scaffoldBackground,
                      title: AppBarTitle(
                          "Enter ${widget.mediaType!.title.capitalize()} URL"))),
              Padding(
                padding: const EdgeInsets.all(12.0),
                child: SunnyForm.builder("videoUrl",
                    validate: ((form) {
                      final uri = form.get(mediaType!.fieldPath)?.toString();
                      if (uri == null || uri.isNotNullOrBlank) {
                        return [
                          ValidationError(
                              path: mediaType!.fieldPath,
                              message: "You must paste a link")
                        ];
                      } else if (!uri
                          .toLowerCase()
                          .startsWith("https://muse.ai")) {
                        return [
                          ValidationError(
                              path: mediaType!.fieldPath,
                              message:
                                  "Invalid link.  We only support muse.ai now")
                        ];
                      } else {
                        return [];
                      }
                    }),
                    form: (context) => MModelForm(context, fact as MModel,
                            mtype: (fact as MModel).mtype,
                            validators: {
                              mediaType!.fieldPath: (value) {
                                if (value.toString().isNullOrBlank) {
                                  return "You must paste a link";
                                } else if (mediaType ==
                                        FlutterMediaTypes.videoExt &&
                                    value
                                            ?.toString()
                                            .toLowerCase()
                                            .startsWith("https://muse.ai") !=
                                        true) {
                                  return "Invalid link. We currently only support links from muse.ai";
                                } else {
                                  return null;
                                }
                              }
                            }),
                    builder: (context, form) {
                      return Layout.column().build([
                        SunnyFormField<String>(
                          label: "URL",
                          attribute: mediaType!.fieldPath,
                          isRequired: true,
                          builder: (context, state) {
                            return TextControl<String>.ofFormField(
                              state,
                              keyboardType: TextInputType.url,
                              isRequired: true,
                            );
                          },
                        ),
                        if (widget.mediaType == FlutterMediaTypes.videoExt)
                          TimeSpanTypeaheadFormField(
                            label: "Starting time",
                            controller: form,
                            showIcon: true,
                            placeholder: "eg. 00:30",
                            attribute: VideoContentPaths.videoLength,
                            isRequired: false,
                          ),
                        SunnyFormSubmit(
                          onPressed: (context) async {
                            final errs =
                                await form.validate(skipIfUnmodified: false);
                            if (errs.isEmpty) {
                              if (mediaType == FlutterMediaTypes.videoExt) {
                                /// For now, we're going to brute force the start time onto the URL
                                final length = form
                                    .path(VideoContentPaths.videoLength)
                                    ?.toDuration();

                                if (length != null) {
                                  final video = fact as IVideoContent;
                                  video.videoUrl = video.videoUrl! +
                                      "?start=${length.inSeconds}";
                                  log.info("Start at ${length.inSeconds}");
                                  video.videoLength = null;
                                }
                              }
                              Navigator.pop(context, fact);
                            }
                          },
                          onCompletion: null,
                        ),
                      ]);
                    }),
              ),
            ]),
          ],
        ),
      ),
    );
  }
}
