@JS()
library muse_embed_web;

import 'dart:html' as html;

import 'package:flutter/widgets.dart';
import 'package:js/js.dart';
import 'package:sunny_dart/sunny_dart.dart';
import 'package:sunny_essentials/shims/universal_ui.dart';
import 'package:sunny_essentials/state.dart';

import 'muse_common.dart';

class MusePlayer {}

@JS()
external MusePlayer embedMuseVideo(
    String id, double width, int start, html.ShadowRoot? shadowRoot);

class Muse extends StatefulWidget {
  final String museId;
  final double width;
  final int start;

  const Muse(
      {Key? key, required this.museId, this.start = 0, required this.width})
      : super(key: key);

  factory Muse.ofUrl(String url, {required double width}) {
    final args = MuseArgs.ofUri(url);
    return Muse(
      museId: args.museId,
      width: width,
      start: args.start,
    );
  }

  @override
  _MuseState createState() => _MuseState();
}

class _MuseState extends BaseState<Muse> {
  MusePlayer? player;
  html.DivElement? _element;
  html.ShadowRoot? _shadow;

  @override
  void initState() {
    super.initState();
    _element = html.DivElement()
      ..id = "muse_${widget.museId}_${widget.width}"
      ..classes = ["muse_${widget.museId}_${widget.width}"]
      ..setAttribute("data-video", widget.museId)
      ..setAttribute("data-width", widget.width.toInt().toString());

    ui.platformViewRegistry
        .registerViewFactory(elementId, (int viewId) => _element);

    /// Manually override what flutter does because we need to be able to access
    /// and pass in the shadow root
    _shadow = _element!.attachShadow({'mode': 'open'});
    initializePlayer();
  }

  @override
  void dispose() {
    super.dispose();
  }

  String get elementId =>
      'muse-${widget.museId}-${widget.width}-${widget.start}';

  void initializePlayer([int count = 0]) async {
    MusePlayer player;
    if (count >= 10) {
      throw "Unable to load muse player in 10 tries";
    }
    try {
      player = await embedMuseVideo(
          widget.museId, widget.width, widget.start, _shadow);
      log.warning("Found player!");
      // setState(() {
      this.player = player;
      // });

    } catch (e, stack) {
      log.severe("Error loading muse player: $e", e, stack);
      Future.delayed(500.ms, () => initializePlayer(count + 1));
    }
  }

  @override
  Widget build(BuildContext context) {
    return Center(
        child: HtmlElementView(key: Key(elementId), viewType: elementId));
  }
}

// class _MuseState extends State<Muse> {
//   Future<MusePlayer> loadMuseVideo(String museId, double width) async {
//
//   }
//
//   Widget museWidget(String museId, double width) {
//     return HtmlElementView(
//       data: '<div id="muse_${museId}">Loading muse video...</div>',
//       key: Key("muse_${museId}"),
//     );
//   }
//
//   @override
//   Widget build(BuildContext context) {
//     return
//   }
// }

// class Muse
