import 'package:dartxx/dartxx.dart';
import 'package:meta_forms/meta_forms.dart';
import 'package:sunny_forms/meta/meta_property_handler.dart';
import 'package:sunny_service_stubs/meta.dart';

/// Used to render embedded schemas. Extending classes pass in [embedRef], and this class automatically identifies
/// the correct properties.
abstract class BaseEmbedPropertyHandler extends MetaPropertyHandler {
  final MSchemaRef embedRef;
  IMSchemaDefinition? embedSchema;

  BaseEmbedPropertyHandler(IMSchemaService _, this.embedRef) {
    _.getSchema(embedRef).then((schema) {
      embedSchema = schema;
    });
  }

  @override
  List<HandledPaths> acceptProperties(
    JsonPath? basePath,
    IMSchemaDefinition definition,
    Map<JsonPath, IMSchemaProperty> available,
  ) =>
      [
        ...available.values
            .where(
                (final p) => p is IEmbeddedProperty && p.embedRef == embedRef)
            .map((p) {
          return HandledPaths.ofProperty(p, basePath!, this);
        }),
      ];
}
