import 'package:flutter/widgets.dart';
import 'package:meta_forms/meta_forms.dart';
import 'package:sunny_dart/sunny_dart.dart';
import 'package:sunny_forms/meta/meta_property_handler.dart';
import 'package:sunny_forms/sunny_form_field.dart';
import 'package:sunny_service_stubs/options.dart';

class HiddenPropertyHandler extends MetaPropertyHandler {
  /// A list of paths that should be ignored
  final Set<JsonPath> ignoredPaths;

  HiddenPropertyHandler(Set ignoredPaths)
      : ignoredPaths = {...ignoredPaths.map((p) => JsonPath.of(p))};

  @override
  List<HandledPaths> acceptProperties(
    JsonPath? basePath,
    IMSchemaDefinition definition,
    Map<JsonPath, IMSchemaProperty> available,
  ) {
    final foundPaths = available.values.where((prop) =>
        ignoredPaths.any((ignored) => prop.jsonPath.startsWith(ignored)));
    return [
      if (foundPaths.isNotEmpty)
        HandledPaths.ofProperties(foundPaths, basePath!, this),
    ];
  }

  @override
  Iterable<Widget> renderProperty(
          BuildContext context,
          IMetaFormContext metaForm,
          HandledPaths paths,
          SunnyFormFieldState state,
          {Key? key}) =>
      [];
}
