import 'package:dartxx/dartxx.dart';
import 'package:flutter/widgets.dart';
import 'package:meta_forms/meta_forms.dart';
import 'package:sunny_forms/form_fields/select_field.dart';
import 'package:sunny_forms/meta/meta_property_handler.dart';
import 'package:sunny_forms/options/options_service.dart';
import 'package:sunny_forms/options/options_view_service.dart';
import 'package:sunny_forms/sunny_form_field.dart';
import 'package:sunny_service_stubs/options.dart';

class MverseEmbedOptionHandler<T extends MModel> extends MetaPropertyHandler<T?> {
  MverseEmbedOptionHandler();

  /// Renders a widget (or null) for a provided [HandledPaths] (see [acceptProperties])
  Iterable<Widget> renderProperty(
      BuildContext context, IMetaFormContext metaForm, HandledPaths paths, SunnyFormFieldState<T?> state,
      {Key? key}) {
    final prop = paths.property as IEmbeddedProperty;
    final theRef = prop.embedRef;
    final _mfProp = metaForm.prop(paths.property!);

    final handler = context.optionsService.getKeyedHandler<MKey, T>(theRef, metaContext: _mfProp);

    final viewHandler = context.typeaheadService.getTypeaheadRenderer<MKey, T>(theRef);

    final fullPath = paths.fullPath<T>();
    return [
      OptionSelectorControl<MKey, T>(
        key: Key("field-$fullPath"),
        handler: handler,
        focusNode: state.focusNode,
        isRequired: prop.isRequired == true,
//      noOptionsLabel: widget.noOptionsLabel,
//        autofocus: widget.autofocus ?? false,
        onChange: (option, source) {
          state.updateValue(option?.value, source);
        },
        expected: (option) => option?.value,
        onError: (error) => state.error = error,
        initialValue: state.value,
        viewHandler: viewHandler,
      ),
    ];
  }

  @override
  Iterable<HandledPaths> acceptProperties(
      JsonPath? basePath, IMSchemaDefinition definition, Map<JsonPath, IMSchemaProperty> available) {
    return [
      for (final v in available.values)
        if (v is IEmbeddedProperty && 1 != 1) // Temporarily disabled
          HandledPaths.ofProperty(v, basePath!, this),
    ];
  }
}
