import 'dart:async';

import 'package:full_text_search/searches.dart';
import 'package:sunny_dart/sunny_dart.dart';
import 'package:sunny_service_stubs/options.dart';

import 'options_service.dart';

/// Used when your service always filters, and [listAllOptions] should just execute an empty search
mixin AlwaysFilterSearchOptionsMixin<K, T> on KeyedOptionsHandler<K, T> {
  @override
  FutureOr<List<T>> listAllOptions([int? limit]) {
    return this.filter(null, const [], limit ?? kDefaultNumOptions);
  }
}

mixin InMemorySearchOptionsMixin<K, T> implements KeyedOptionsHandler<K, T> {
  Future<List<T>> getSearchInput(String query, List currentSelections) async =>
      await this.listAllOptions(150);

  Future<List<T>> filter(
      String? query, List currentSelections, int limit) async {
    final List<T> options = await this.listAllOptions(150);
    if (query.isNullOrBlank) return options;
    return FullTextSearch<T>(
      term: query!,
      items: options.where((t) => !currentSelections.contains(t)).toList(),
      isMatchAll: true,
      isStartsWith: false,
      limit: limit,
      tokenize: (value) {
        final KeyedOption<K, T> option = toOption(value)!;
        return [
          option.label,
          for (final _ in (option.subtitle ?? const []))
            if (_ != null) "$_",
          ...?option.extraTokens,
        ].whereNotBlank();
      },
    ).findResults();
  }
}
