import 'dart:async';

import 'package:flutter/widgets.dart';
import 'package:sunny_dart/sunny_dart.dart';
import 'package:sunny_essentials/sunny_essentials.dart';
import 'package:sunny_forms/date_typeahead.dart';
import 'package:sunny_forms/options/options_service.dart';
import 'package:sunny_forms/options/options_view_service.dart';
import 'package:sunny_forms/smart_dates/smart_date_query.dart';
import 'package:sunny_forms/smart_dates/smart_date_service.dart';
import 'package:sunny_sdk_core/mverse.dart';
import 'package:sunny_service_stubs/sunny_service_stubs.dart';

import 'mixins.dart';

///
extension BuildContextSmartDateOptionsExt on BuildContext {
  ///
  SmartDateOptionsHandler get smartDateOptions => Provided.get(this);
}

final smartDateTypeaheadHandler =
    defaultOptionViewHandler<SmartDateQueryResult, SmartDateQueryResult>(
        showExpand: false, noOptionsLabel: "N/A");

class SmartDateOptionsHandler extends OptionsHandler<
        SmartDateQueryResult> //    with RenderingOptionsHandler<SmartDateQueryResult>
    with
        AlwaysFilterSearchOptionsMixin<SmartDateQueryResult,
            SmartDateQueryResult> {
  ///
  final SmartDateService _smartDateService;
  final SmartDateOptionsConfig config;

  final String placeholder;

  SmartDateOptionsHandler(
      OptionsService optionService, SmartDateService smartDateService)
      : this.withContext(
          smartDateService,
          optionService: optionService,
          config: const SmartDateOptionsConfig.flexiDate(),
        );

  SmartDateOptionsHandler.withContext(
    this._smartDateService, {
    String? placeholder,
    required this.config,
    OptionsService? optionService,
  })  : placeholder = placeholder ?? "e.g. birthday, July 1",
        super("smartDateQueryResult") {
    optionService?.register(this);
  }

  @override
  SmartDateQueryResult toKey(value) => value;

  @override
  bool get canShowAll => config.defaultDates.isNotNullOrEmpty;

  TypeaheadHandler<SmartDateQueryResult, SmartDateQueryResult>
      get typeaheadHandler {
    return defaultOptionViewHandler(
      placeholder: placeholder,
      focusMode: TypeaheadFocusMode.showAll,
      showExpand: false,
//      config.defaultDates.isNotEmpty,
    );
  }

  @override
  Future<List<SmartDateQueryResult>> filter(
      String? query, List currentSelections, int limit) async {
    if (query.isNullOrBlank) {
      final location = sunnyLocalization.userLocation;
      final defaultResults = [
        /// Then, process the expressions
        for (final expression
            in config.defaultDates.whereType<DateExpression>())
          SmartDateQueryResult.ofDateTime(expression.dateProducer(location!),
                  icon: expression.iconName, overrideLabel: expression.title)
              .withDateRef(expression),

//        /// First, process all the MetaDates
//        for (final metaDate in config.defaultDates.whereType<MetaDateRef>())
//          for (final factDate in await _smartDateService.resolveMetaDateResults(
//              refs: [metaDate],
////              contact: config.contact,
////              facts: config.facts,
//              location: location))
//            factDate.withDateRef(metaDate),

        /// Lastly, the query expressions
        for (final query in config.defaultDates.whereType<SimpleDateQuery>())
          for (final queryResult in await _filter(query.query, [], limit))
            queryResult.withDateRef(query),
      ];
      return defaultResults;
    } else {
      return await _filter(query!, currentSelections, limit);
    }
  }

  Future<List<SmartDateQueryResult>> _filter(
      String query, List currentSelections, int limit) async {
    final location = sunnyLocalization.userLocation;
    final results = await _smartDateService.parseDates(
      query,
      isFlexiDate: config.isFlexiDate,
//      contact: config.contact,
//      facts: config.facts,
      location: location,
    );
    return results;
  }
  //
  // bool get showImmediateOptions => false;
  //
  // Icon get inputIcon {
  //   return NamedIcons["calendar"].icon;
  // }

  SmartDateOptionsHandler withInputContext(SmartDateOptionsConfig config) {
    return copy(config: config);
  }

  @override
  Option<SmartDateQueryResult>? toOption(SmartDateQueryResult? value) {
    return value?.toOption();
  }

  @override
  KeyedAdhocBuilder<SmartDateQueryResult, SmartDateQueryResult>?
      get adhocOptionBuilder => null;

  SmartDateOptionsHandler copy({
    SmartDateService? smartDateService,
    SmartDateOptionsConfig? config,
    String? placeholder,
  }) {
    return SmartDateOptionsHandler.of(
      smartDateService: smartDateService ?? this._smartDateService,
      config: config ?? this.config,
      placeholder: placeholder ?? this.placeholder,
    );
  }

  SmartDateOptionsHandler.of({
    required this.config,
    required this.placeholder,
    required SmartDateService smartDateService,
  })   : _smartDateService = smartDateService,
        super(Definitions.smartDateQueryResult);

  @override
  IKeyedOptionsHandler<KK, VV> cast<KK, VV>() {
    return this as IKeyedOptionsHandler<KK, VV>;
  }

//  @override
//  Widget buildOption(BuildContext context, SmartDateQueryResult value, {VoidCallback onTap}) {
//    return value.toOption();
//  }
}

extension SmartDateQueryResultDateRefExtensions on SmartDateQueryResult {
  SmartDateQueryResult withDateRef(DateRef ref) {
    return DelegatingSmartDateQueryResult(
      this,
      title: ref.title,
      subtitle: ref.subtitle.toStringList(),
      icon: ref.iconName,
      selection: ref.selection,
    );
  }
}
