import 'package:dartxx/dartxx.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:sunny_platform_widgets/sunny_platform_widgets.dart';
import 'package:sunny_forms/sunny_colors.dart';
import 'package:sunny_forms/sunny_text_field.dart';

class PlatformAppBarTextField extends PlatformWidgetBase {
  final TextEditingController? controller;
  final FocusNode? focusNode;
  final String? label;
  final String? attribute;
  final IconData? prefixIcon;
  final EdgeInsets? iosPadding;
  final bool? autofocus;
  final TextInputType? keyboardType;
  final bool? autocorrect;
  final TextInputAction? inputAction;
  final ValueChanged<String>? onSubmitted;

  PlatformAppBarTextField({
    Key? key,
    this.keyboardType,
    this.attribute,
    this.label,
    this.prefixIcon,
    this.iosPadding,
    this.onSubmitted,
    this.focusNode,
    this.autofocus,
    this.autocorrect,
    this.inputAction,
    this.controller,
    extraOptions,
  }) : super(key: key, extraOptions: extraOptions);

  @override
  Widget createMaterialWidget(BuildContext context) {
    return Material(
      elevation: 5,
      child: Container(
        padding: SunnyMargins.all,
        alignment: Alignment.centerLeft,
        color: SunnyColors.navBarBackground,
        child: TextField(
//          keyboardType: keyboardType,
          textInputAction: inputAction,
          expands: false,
          onSubmitted: onSubmitted,

          maxLines: 1,
          autofocus: autofocus == true,
          focusNode: focusNode,
          autocorrect: autocorrect ?? false,
          controller: controller,
          style: TextStyle(
            color: Theme.of(context).brightness == Brightness.light
                ? Colors.black.withOpacity(0.9)
                : Colors.white.withOpacity(0.9),
            fontSize: 20.0,
          ),
          decoration: InputDecoration(
            floatingLabelBehavior: FloatingLabelBehavior.never,
            icon: Icon(prefixIcon),
            isDense: false,
//          prefix: Icon(prefixIcon, size: 20),
            hintText: label,
//          fillColor: Theme.of(context).brightness == Brightness.light ? Colors.white30 : Colors.black38,
            hintStyle: TextStyle(
              color: Theme.of(context).brightness == Brightness.light
                  ? Colors.black38
                  : Colors.white30,
              fontSize: 20.0,
            ),

            contentPadding: EdgeInsets.zero,
            border: InputBorder.none,
            disabledBorder: InputBorder.none,
            focusedBorder: InputBorder.none,
            enabledBorder: InputBorder.none,
            errorBorder: InputBorder.none,
          ),
        ),
      ),
    );
  }

  @override
  Widget createCupertinoWidget(BuildContext context) {
    final padLeft = prefixIcon == null ? 10.0 : 6.0;
    return Container(
      padding: iosPadding ?? SunnyMargins.all,
      alignment: Alignment.centerLeft,
      child: SunnyTextField(
        autofocus: autofocus == true,
        attribute: JsonPath.parsed(attribute!),
        expands: false,
        maxLines: 1,
        cupertino: (context, _) => CupertinoTextFieldData(
            prefix: prefixIcon == null
                ? null
                : Container(
                    padding: EdgeInsets.only(left: padLeft),
                    child: Icon(prefixIcon,
                        size: 20, color: SunnyColors.inputControls)),
            clearButtonMode: OverlayVisibilityMode.editing,
            autocorrect: autocorrect == true,
            expands: false,
            maxLines: 1,
            onSubmitted: onSubmitted,
            textInputAction: inputAction,
            decoration: BoxDecoration(
                borderRadius: BorderRadius.circular(12),
                color: SunnyColors.inputBackground),
            style: SunnyTextStyles.appBarInput,
            padding:
                EdgeInsets.only(left: padLeft, top: 8, bottom: 8, right: 6),
            controller: controller,
            placeholder: label,
            focusNode: focusNode,
            placeholderStyle: SunnyTextFieldStyles.inputPlaceholder),
      ),
    );
  }
}
