import 'package:flutter/widgets.dart';
import 'package:sunny_core_widgets/sunny_core_widgets.dart';
import 'package:sunny_forms/text_widgets.dart';
import 'package:sunny_forms/widget_extensions.dart';

abstract class Listable {
  String? get title;

  List? get subtitle;

  String? get icon;

  /// Action text to display in the trailing section;
  dynamic get trailing;

  bool get isCard;

  factory Listable({
    String? title,
    List? subtitle,
    String? icon,
    dynamic trailing,
    bool isCard = true,
  }) =>
      _Listable(title, subtitle, icon, trailing, isCard);
}

class _Listable implements Listable {
  final String? title;
  final List? subtitle;
  final String? icon;
  final dynamic trailing;
  final bool isCard;

  _Listable(this.title, this.subtitle, this.icon, this.trailing, this.isCard);
}

mixin ListableMixin implements Listable {
  List? get subtitle => const [];

  String? get icon => null;

  /// Action text to display in the trailing section;
  dynamic get trailing => null;

  bool get isCard => false;

  Widget toListTile(BuildContext context,
      {Key? key, bool? isCard, double iconSize = 30, VoidCallback? onTap}) {
    Widget? trailing;
    final thisTrailing = this.trailing;
    if (thisTrailing != null) {
      if (thisTrailing is Widget) trailing = thisTrailing;
      if (thisTrailing is String) trailing = LinkText(thisTrailing);
    }
    return PlatformListTile(
      key: key,
      useShadow: isCard ?? true,
      // dense: isCard == false,
      margin: isCard == false ? EdgeInsets.zero : null,
      onTap: onTap == null ? null : (context) => onTap(),
      title: TileTitle.of(title),
      subtitle: subtitle.toColumn(),
      leading: context.findIcon(icon!)?.toListSelectOption(),
      trailing: trailing,
    );
  }
}
