import 'package:collection_diff/diff_equality.dart';
import 'package:dartxx/dartxx.dart';
import 'package:sunny_sdk_core/sunny_sdk_core.dart';

class RecordKeyLookup<V> with DiffDelegateMixin {
  final Map<RecordKey, V> _map;

  RecordKeyLookup(this._map);

  RecordKeyLookup.empty() : _map = const {};

  V? operator [](RecordKey key) {
    return _map[key];
  }

  int get length => _map.length;
  Iterable<V> get values => _map.values;
  Iterable<RecordKey> get keys => _map.keys;

  @override
  get diffKey {
    return {
      ..._map.map((k, vs) {
        return MapEntry(k, [
          if (vs is Iterable)
            for (final v in vs)
              if (v is DiffDelegate) v.diffKey else v
        ]);
      })
    };
  }

  @override
  bool operator ==(Object other) =>
      identical(this, other) ||
      other is RecordKeyLookup && other.diffKey == diffKey;

  @override
  int get hashCode => diffKey.hashCode;

  @override
  String toString() {
    return 'RecordKeyLookup{ \n ${_map.mapEach((k, v) {
      return "  - $k => $v \n";
    })} }';
  }
}

RecordKeyLookup<T> recordKeyLookup<T>(Map<RecordKey, T> records) =>
    RecordKeyLookup(records);
