import 'package:dartxx/dartxx.dart';
import 'package:flutter/widgets.dart';
import 'package:flutter_mobx/flutter_mobx.dart';
import 'package:flutter_staggered_grid_view/flutter_staggered_grid_view.dart';
import 'package:sunny_forms/builders/cross_fade.dart';
import 'package:sunny_forms/standard_column.dart';
import 'package:sunny_forms/sunny_list/animated_grid.dart';
import 'package:sunny_forms/sunny_list/sunny_animated_list.dart';
import 'package:sunny_forms/sunny_list/sunny_list_base.dart';

class SunnyGridState<T> extends SunnyListBaseState<T?> {
  final double mainAxisSpacing;
  final double crossAxisSpacing;
  final int crossAxisCount;

  // final StaggeredTileBuilder<T?> staggeredBuilder;

  GlobalKey<AnimatedGridState>? globalKey;

  SunnyGridState.ofFixedExtent({
    /// GRID PROPERTIES
    required this.mainAxisSpacing,
    required this.crossAxisSpacing,
    required this.crossAxisCount,
    required double tileHeight,
    required int tileWidthUnit,
  });

  SunnyGridState({
    /// GRID PROPERTIES
    required this.mainAxisSpacing,
    required this.crossAxisSpacing,
    required this.crossAxisCount,
  });

  @override
  initState() {
    super.initState();
    globalKey =
        GlobalKey<AnimatedGridState>(debugLabel: "sunny-list-gk-${widget.id}");
  }

  @override
  dispose() {
    super.dispose();
  }

  @override
  void insertItem(State? state, int? index, T? itemToInsert) {
    log.info("$debugName had ADD[$index]");
    final gridState = state as AnimatedGridState;
    gridState.insertItem(index);
  }

  @override
  void removeItem(State? state, int? index, T? itemToRemove) {
    log.info("$debugName had REMOVE[$index]");
    final gridState = state as AnimatedGridState;
    gridState.removeItem(index, (context, animation) {
      return animated(
        widget.dismissAnimationType,
        name: "remove-$index",
        reversed: true,
        animation: animation,
        child: itemBuilder(context, itemToRemove, index, animation),
      );
    });
  }

  @override
  void updateItem(State state, int index, T? oldItem, T? newItem) {
    log.info("$debugName had UPDATE[$index] (ignoring)");
  }

  @override
  void moveItem(State state, int fromIndex, int toIndex, T? itemToMove) {
    log.info("$debugName had MOVE[$fromIndex] to[$toIndex]");
  }

  @override
  Widget build(BuildContext context) {
    final animatedGrid = AnimatedGrid(
      primary: widget.primary ?? false,
      reverse: widget.reverse,
      key: globalKey,
      controller: widget.scrollController,
      padding: widget.padding ?? EdgeInsets.zero,
      shrinkWrap: widget.shrinkWrap ?? true,
      initialItemCount: trackedList!.length,
      itemBuilder: buildCompleteItem,
      mainAxisSpacing: mainAxisSpacing,
      crossAxisCount: crossAxisCount,
      crossAxisSpacing: crossAxisSpacing,
      //
      // staggeredBuilder: (int index) {
      //   final T? item = trackedList!.tryGet(index);
      //   return staggeredBuilder(item, index);
      // },
    );

    return Observer(
      key: Key("${widget.id}-grid-empty"),
      builder: (_) {
        return column([
          if (widget.header != null)
            Observer(
                key: Key("${widget.id}-grid-header"),
                builder: (_) => widget.header!(context, this)),
          crossFade(
            first: animatedGrid,
            second: widget.emptyState?.call(context, this) ?? emptyBox,
            showFirst: isDisplayed,
          ),
        ]);
      },
    );
  }
}
