import 'package:flutter/widgets.dart';
import 'package:flutter_mobx/flutter_mobx.dart';
import 'package:mobx/mobx.dart' hide ObservableMap, ListChange;
import 'package:sunny_forms/builders/cross_fade.dart';
import 'package:sunny_forms/standard_column.dart';
import 'package:sunny_forms/sunny_list/sunny_animated_list.dart';

import 'sunny_list_base.dart';

/// A stateful widget that binds to an [ObservableList] and animates the changes tracked to it

class SunnyListState<T> extends SunnyListBaseState<T> {
  @override
  GlobalKey<AnimatedListState>? globalKey;

  @override
  void initState() {
    super.initState();
    globalKey = GlobalKey<AnimatedListState>(debugLabel: widget.id);
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  void insertItem(State? state, int? index, T? itemToInsert) {
    globalKey!.currentState!.insertItem(index!);
  }

  @override
  void removeItem(State? state, int? index, T itemToRemove) {
    globalKey!.currentState!.removeItem(index!, (context, animation) {
      return animated(
        widget.dismissAnimationType,
        name: "remove-$index",
        reversed: true,
        animation: animation,
        child: itemBuilder(context, itemToRemove, index, animation),
      );
    });
  }

  @override
  void updateItem(State state, int index, T? oldItem, T? newItem) {}

  @override
  void moveItem(State state, int fromIndex, int toIndex, T? itemToMove) {}

  @override
  Widget build(BuildContext context) {
    final list = () => AnimatedList(
        primary: widget.primary ?? false,
        reverse: widget.reverse,
        key: globalKey,
//        controller: widget.scrollController,
        padding: widget.padding ?? EdgeInsets.zero,
        shrinkWrap: widget.shrinkWrap ?? true,
        initialItemCount: trackedList!.length,
        itemBuilder: buildCompleteItem);
    return Observer(
      key: Key("${widget.id}-isEmpty"),
      builder: (_) {
        return column(
          [
            if (widget.header != null)
              Observer(
                key: Key("${widget.id}-header"),
                builder: (_) => widget.header!(context, this),
              ),
            list(),
            crossFade(
              first: emptyBox,
              second: widget.emptyState?.call(context, this) ?? emptyBox,
              showFirst: isDisplayed,
            )
          ],
        );
      },
    );
  }
}
