import 'package:flutter/widgets.dart';
import 'package:flutter_mobx/flutter_mobx.dart';
import 'package:sunny_essentials/container.dart';
import 'package:sunny_forms/sunny_list/sunny_animated_list.dart';
import 'package:sunny_forms/sunny_list/sunny_list_base.dart';

class SunnySliverAnimatedListState<T> extends SunnyListBaseState<T?> {
  GlobalKey<SliverAnimatedListState>? globalKey;
  int? initialCount;

  @override
  void initState() {
    super.initState();
    globalKey = GlobalKey<SliverAnimatedListState>();
    if (widget.header != null) {
      log.warning("Sliver list doesn't support headers");
    }
    initialCount = trackedList!.length;
  }

  @override
  void insertItem(State? state, int? index, T? itemToInsert) {
    globalKey!.currentState?.insertItem(index!);
  }

  @override
  void removeItem(State? state, int? index, T? itemToRemove) {
    globalKey!.currentState?.removeItem(index!, (context, animation) {
      return animated(
        widget.dismissAnimationType,
        name: "remove-$index",
        reversed: true,
        animation: animation,
        child: itemBuilder(context, itemToRemove, index, animation),
      );
    });
  }

  @override
  void updateItem(State state, int index, T? oldItem, T? newItem) {}

  @override
  void moveItem(State state, int fromIndex, int toIndex, T? itemToMove) {
    log.info('$debugName had MOVE[$fromIndex] to[$toIndex]');
  }

  @override
  Widget build(BuildContext context) {
    return Observer(
      key: Key('sunny-list-state-observer-${widget.id}'),
      builder: (_) {
        return isDisplayed
            ? SliverAnimatedList(
                key: globalKey,
                initialItemCount: trackedList!.length,
                itemBuilder: buildCompleteItem,
              )
            : SliverToBoxAdapter(
                child: widget.emptyState?.call(context, this) ?? emptyBox);
      },
    );
  }
}
