// ignore_for_file: deprecated_member_use
import 'package:dartxx/dartxx.dart';
import 'package:flutter/material.dart';
import 'package:sunny_platform_widgets/sunny_platform_widgets.dart';
import 'package:sunny_dart/sunny_get.dart';
import 'package:sunny_essentials/state.dart';
import 'package:sunny_forms/notifier.dart';
import 'package:sunny_forms/standard_column.dart';
import 'package:sunny_forms/sunny_colors.dart';
import 'package:sunny_forms/sunny_form_controller.dart';
import 'package:sunny_forms/sunny_form_field.dart';
import 'package:sunny_forms/sunny_form_functions.dart';

class SunnyTextField<T extends Object> extends StatefulWidget {
  final PlatformBuilder<MaterialTextFieldData> material;
  final PlatformBuilder<CupertinoTextFieldData> cupertino;
  final JsonPath attribute;
  final bool enabled;
  final bool obscureText;
  final int? minLength;
  final bool expands;
  final int? minLines;
  final int? maxLines;
  final FocusNode? focusNode;
  final ValueChanged<T?>? onSubmitted;
  final bool? isRequired;
  final bool autovalidate;
  final List<ValidatorFn<T?>> validators;
  final FieldConverter<T>? converter;
  final dynamic label;
  final dynamic placeholder;
  final bool autofocus;
  final bool? autocorrect;
  final TextEditingController? controller;
  final FormValueChanged<T?>? onChange;
  final TextInputType? keyboardType;
  final TextInputAction? textInputAction;
  final TextCapitalization? textCapitalization;
  final T? initialValue;

  final TextStyle? placeholderStyle;

  SunnyTextField.formField(
      {Key? key,
      required this.attribute,
      this.obscureText = false,
      this.label,
      this.keyboardType,
      this.minLength,
      this.validators = const [],
      this.isRequired,
      this.autovalidate = false,
      this.initialValue,
      this.onChange,
      this.focusNode,
      this.textCapitalization,
      this.onSubmitted,
      this.autocorrect,
      this.textInputAction,
      this.placeholder,
      this.converter,
      this.controller,
      this.autofocus = false,
      this.placeholderStyle,
      this.expands = false,
      this.minLines,
      this.maxLines,
      this.enabled = true})
      : cupertino = ((context, _) => CupertinoTextFieldData(
            expands: false,
            padding: EdgeInsets.zero,
            decoration: BoxDecoration(
                border: Border.all(style: BorderStyle.none, width: 0)))),
        material = ((context, _) => MaterialTextFieldData(expands: false)),
        super(key: key ?? Key("$attribute-input"));

  SunnyTextField(
      {Key? key,
      required this.attribute,
      this.material = _emptyMaterial,
      this.cupertino = _emptyCupertino,
      this.obscureText = false,
      this.label,
      this.keyboardType,
      this.minLength,
      this.validators = const [],
      this.isRequired,
      this.autovalidate = false,
      this.initialValue,
      this.onChange,
      this.focusNode,
      this.onSubmitted,
      this.autocorrect,
      this.textInputAction,
      this.textCapitalization,
      this.placeholder,
      this.converter,
      this.controller,
      this.autofocus = false,
      this.placeholderStyle,
      this.expands = false,
      this.minLines,
      this.maxLines,
      this.enabled = true})
      : super(key: key ?? Key("$attribute-input"));

  @override
  State<StatefulWidget> createState() => _SunnyTextFieldState<T>();
}

MaterialTextFieldData _emptyMaterial(BuildContext context, target) {
  return MaterialTextFieldData();
}

CupertinoTextFieldData _emptyCupertino(BuildContext context, target) {
  return CupertinoTextFieldData();
}

class _SunnyTextFieldState<T extends Object>
    extends BaseState<SunnyTextField<T>> {
  TextEditingController? controller;

  @override
  void initState() {
    super.initState();
    final startValue = widget.initialValue?.toString() ?? "";

    controller = widget.controller ??
        TextEditingController.fromValue(
          TextEditingValue(
            text: startValue,
            selection: TextSelection.collapsed(offset: startValue.length),
          ),
        );
  }

  @override
  void dispose() {
    if (widget.controller == null) {
      controller?.dispose();
    }
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final sunnyForm = SunnyForm.of(context, nullOk: true);

    var intlMessage = (widget.placeholder)?.toString();
    var androidData = widget.material(context, platform(context));
    var androidDecoration = androidData.decoration?.copyWith(
            hintText: androidData.decoration?.hintText ?? intlMessage) ??
        InputDecoration(hintText: intlMessage);

    final intl = sunny.get<SunnyIntl>(context: context);
    return FormField<T>(
      validator: (input) {
        if (widget.isRequired == true && input?.toString().isNotEmpty != true) {
          return intl.required;
        }
        if (widget.minLength != null &&
            widget.minLength! > 0 &&
            (input?.toString() ?? "").length < widget.minLength!) {
          return intl.minLength(widget.minLength!);
        }
        for (final validator in widget.validators) {
          final result = validator(input);
          if (result != null) {
            return result;
          }
        }
        return null;
      },
      onSaved: (value) {},
      enabled: widget.enabled,
      initialValue: widget.initialValue,
      autovalidateMode: widget.autovalidate == true
          ? AutovalidateMode.onUserInteraction
          : AutovalidateMode.disabled,
      builder: (state) {
        return column([
          if (state.errorText?.isNotEmpty == true)
            Padding(
              padding: EdgeInsets.only(bottom: 5),
              child: Text(state.errorText!, style: SunnyTextStyles.errorText),
            ),
          PlatformTextField(
            autofocus: widget.autofocus,
            autocorrect: widget.autocorrect ?? false,
            controller: controller,
            expands: widget.expands,
            minLines: widget.minLines,
            maxLines: widget.maxLines,
            enabled: widget.enabled,
            textCapitalization:
                widget.textCapitalization ?? TextCapitalization.sentences,
            textInputAction: widget.textInputAction,
            onSubmitted: (string) {
              if (widget.onSubmitted != null) {
                final T? value = widget.converter != null
                    ? widget.converter!(string)
                    : string as T;

                widget.onSubmitted?.call(value);
              } else {
                FocusScope.of(context).requestFocus(FocusNode());
              }
            },
            onEditingComplete: () {},
            focusNode: widget.focusNode,
            keyboardAppearance: Brightness.light,
            keyboardType: widget.keyboardType,
            obscureText: widget.obscureText,
            cupertino: (context, _) => mergeCupertinoTextFieldData(
                _iosDefaultTextData(intlMessage),
                widget.cupertino(context, platform(context))),
            material: (context, _) => MaterialTextFieldData(
              decoration: androidDecoration,
              expands: widget.expands,
              focusNode: widget.focusNode,
              minLines: widget.minLines,
              maxLines: widget.maxLines,
            ),
            onChanged: (String value) {
              try {
                final T? converted = widget.converter != null
                    ? widget.converter!(value)
                    : value as T;
                state.didChange(converted);

                if (widget.onChange != null) {
                  widget.onChange!(converted, AttributeSource.control);
                }

                if (sunnyForm != null) {
                  sunnyForm.set(widget.attribute, value);
                }
              } catch (e, stack) {
                log.info(e);
                log.info(stack);
              }
            },
          ),
          if (state.errorText?.isNotEmpty == true)
            Padding(
              padding: EdgeInsets.only(bottom: 5),
              child: Text(state.errorText!, style: SunnyTextStyles.errorText),
            ),
        ]);
      },
    );
  }

  CupertinoTextFieldData _iosDefaultTextData(intlMessage) {
    return mergeCupertinoTextFieldData(
        defaultCupertinoTextField(),
        CupertinoTextFieldData(
            expands: widget.expands,
            focusNode: widget.focusNode,
            minLines: widget.minLines,
            maxLines: widget.maxLines,
            autocorrect: widget.autocorrect,
            placeholder: intlMessage?.toString(),
            placeholderStyle: widget.placeholderStyle ??
                SunnyTextFieldStyles.inputPlaceholder));
  }
}

CupertinoTextFieldData mergeCupertinoTextFieldData(
        CupertinoTextFieldData? defaults, CupertinoTextFieldData? overrides) =>
    CupertinoTextFieldData(
        widgetKey: overrides?.widgetKey ?? defaults?.widgetKey,
        controller: overrides?.controller ?? defaults?.controller,
        focusNode: overrides?.focusNode ?? defaults?.focusNode,
        decoration: overrides?.decoration ?? defaults?.decoration,
        padding: overrides?.padding ?? defaults?.padding,
        placeholder: overrides?.placeholder ?? defaults?.placeholder,
        placeholderStyle:
            overrides?.placeholderStyle ?? defaults?.placeholderStyle,
        prefix: overrides?.prefix ?? defaults?.prefix,
        prefixMode: overrides?.prefixMode ?? defaults?.prefixMode,
        suffix: overrides?.suffix ?? defaults?.suffix,
        suffixMode: overrides?.suffixMode ?? defaults?.suffixMode,
        clearButtonMode:
            overrides?.clearButtonMode ?? defaults?.clearButtonMode,
        keyboardType: overrides?.keyboardType ?? defaults?.keyboardType,
        textInputAction:
            overrides?.textInputAction ?? defaults?.textInputAction,
        textCapitalization:
            overrides?.textCapitalization ?? defaults?.textCapitalization,
        style: overrides?.style ?? defaults?.style,
        textAlign: overrides?.textAlign ?? defaults?.textAlign,
        autofocus: overrides?.autofocus ?? defaults?.autofocus,
        obscureText: overrides?.obscureText ?? defaults?.obscureText,
        autocorrect: overrides?.autocorrect ?? defaults?.autocorrect,
        maxLines: overrides?.maxLines ?? defaults?.maxLines,
        maxLength: overrides?.maxLength ?? defaults?.maxLength,
        maxLengthEnforcement:
            overrides?.maxLengthEnforcement ?? defaults?.maxLengthEnforcement,
        onChanged: overrides?.onChanged ?? defaults?.onChanged,
        onEditingComplete:
            overrides?.onEditingComplete ?? defaults?.onEditingComplete,
        onSubmitted: overrides?.onSubmitted ?? defaults?.onSubmitted,
        inputFormatters:
            overrides?.inputFormatters ?? defaults?.inputFormatters,
        enabled: overrides?.enabled ?? defaults?.enabled,
        cursorWidth: overrides?.cursorWidth ?? defaults?.cursorWidth,
        cursorRadius: overrides?.cursorRadius ?? defaults?.cursorRadius,
        cursorColor: overrides?.cursorColor ?? defaults?.cursorColor,
        keyboardAppearance:
            overrides?.keyboardAppearance ?? defaults?.keyboardAppearance,
        scrollPadding: overrides?.scrollPadding ?? defaults?.scrollPadding);

mergeMaterialTextFieldData(
        MaterialTextFieldData? defaults, MaterialTextFieldData? overrides) =>
    MaterialTextFieldData(
        widgetKey: overrides?.widgetKey ?? defaults?.widgetKey,
        controller: overrides?.controller ?? defaults?.controller,
        focusNode: overrides?.focusNode ?? defaults?.focusNode,
        decoration: overrides?.decoration ?? defaults?.decoration,
        keyboardType: overrides?.keyboardType ?? defaults?.keyboardType,
        textInputAction:
            overrides?.textInputAction ?? defaults?.textInputAction,
        textCapitalization:
            overrides?.textCapitalization ?? defaults?.textCapitalization,
        style: overrides?.style ?? defaults?.style,
        textAlign: overrides?.textAlign ?? defaults?.textAlign,
        textDirection: overrides?.textDirection ?? defaults?.textDirection,
        autofocus: overrides?.autofocus ?? defaults?.autofocus,
        obscureText: overrides?.obscureText ?? defaults?.obscureText,
        autocorrect: overrides?.autocorrect ?? defaults?.autocorrect,
        maxLines: overrides?.maxLines ?? defaults?.maxLines,
        maxLength: overrides?.maxLength ?? defaults?.maxLength,
        maxLengthEnforcement:
            overrides?.maxLengthEnforcement ?? defaults?.maxLengthEnforcement,
        onChanged: overrides?.onChanged ?? defaults?.onChanged,
        onEditingComplete:
            overrides?.onEditingComplete ?? defaults?.onEditingComplete,
        onSubmitted: overrides?.onSubmitted ?? defaults?.onSubmitted,
        inputFormatters:
            overrides?.inputFormatters ?? defaults?.inputFormatters,
        enabled: overrides?.enabled ?? defaults?.enabled,
        cursorWidth: overrides?.cursorWidth ?? defaults?.cursorWidth,
        cursorRadius: overrides?.cursorRadius ?? defaults?.cursorRadius,
        cursorColor: overrides?.cursorColor ?? defaults?.cursorColor,
        keyboardAppearance:
            overrides?.keyboardAppearance ?? defaults?.keyboardAppearance,
        scrollPadding: overrides?.scrollPadding ?? defaults?.scrollPadding,
        enableInteractiveSelection: overrides?.enableInteractiveSelection ??
            defaults?.enableInteractiveSelection,
        onTap: overrides?.onTap ?? defaults?.onTap);

CupertinoTextFieldData defaultCupertinoTextField() {
  return CupertinoTextFieldData(
      decoration: BoxDecoration(
          color: sunnyColors.white,
          border: Border(bottom: BorderSide(color: sunnyColors.textLight)),
          shape: BoxShape.rectangle),
      style: SunnyTextFieldStyles.inputText,
      cursorColor: sunnyColors.primaryColor,
      padding: SunnyMargins.textInputPadding,
      textInputAction: TextInputAction.done,
      placeholderStyle: SunnyTextFieldStyles.inputPlaceholder);
}
